import 'dart:convert';

import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/CardComponent/card_builder.dart';
import 'package:bui_component/DrawerComonent/kfDrawer.dart';
import 'package:bui_component/HomeScreenComponent.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/fontSize.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/dash_board_detail_bean.dart';
import 'package:example/Screen/ERP/FilterScreen.dart';
import 'package:example/Screen/ERP/NoInternet.dart';
import 'package:example/Screen/ERP/appUpdate.dart';
import 'package:example/Screen/ThemePickerScreen.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/Utils/class_builder.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import '../main.dart';
import 'ButtonView.dart';

class DashBoardScreen extends KFDrawerContent {
  final Function() onMenuIconPressed;

  DashBoardScreen({this.onMenuIconPressed});

  @override
  _DashBoardScreenState createState() => _DashBoardScreenState();
}

class _DashBoardScreenState extends State<DashBoardScreen> {
  Color theme = AppColors.APP_COLOR_PRIMARY;
  Color secondaryBackgroundColor;

  List<PopupMenuItem> popUpMenu = new List();
  SharedPreferences prefs;

  bool isLoader = true;
  bool isAppUpdate = false;
  num countValue = 2;

  num aspectWidth = 2;

  num aspectHeight = 1;

  List<Dashboard> dashboard = new List();

  bool isGridView = false;

  DashBoardDetailBean dashBoardDetailsBeanResponse;

  var now = DateTime.now();

  @override
  void initState() {
    ClassBuilder.registerClasses();

    colorCodePref();

    callDashBoardAPI();

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    // FocusScope.of(context).requestFocus(new FocusNode());
    return Scaffold(
      body: HomeScreenComponent(
        onMenuPressed: onMenuPressed,
        backgroundColor: theme,
        onPopUpMenuFunction: onPopUpMenuPressed,
        popUpMenu: popUpMenu,
        showFilterView: false,
        onFilterClicked: () {
          Navigator.of(context).push(MaterialPageRoute(
            builder: (context) {
              return FilterScreen(
                primaryColor: theme,
                secondaryColor: secondaryBackgroundColor,
              );
            },
          ));
        },
        child: isLoader
            ? Stack(
                children: [
                  Loader(
                    loaderColor: theme,
                  ),
                ],
              )
            : isAppUpdate
                ? Stack(
                    children: [AppForceUpdate()],
                  )
                : Stack(
                    children: [
                      showGridView,
                      apiCallView,
                    ],
                  ),
      ),
    );
  }

  Widget get showGridView {
    return Container(
      margin: EdgeInsets.only(top: 50),
      child: InkWell(
        onTap: () {
          setState(() {
            // widget.isGridView = !widget.isGridView;

            changeOrderView();
          });
        },
        child: Container(
            alignment: Alignment.topRight,
            padding: EdgeInsets.only(right: 20, top: 20),
            child: Icon(
              !isGridView ? Icons.list : Icons.grid_view,
              semanticLabel:
                  isGridView ? "Change to List View" : "Change to Grid View",
              color: secondaryBackgroundColor ?? theme,
            )),
      ),
    );
  }

  Widget get apiCallView {
    return Container(
        margin: EdgeInsets.only(top: 100),
        padding: EdgeInsets.only(top: 10, bottom: 20),
        child: GridView.count(
          crossAxisCount: countValue,
          childAspectRatio: (aspectWidth / aspectHeight),
          children: List.generate(
              dashBoardDetailsBeanResponse.data.dashboard.length, (index) {
            return BUICard.dashBoardCardDeals(
              cardTitle: dashboard[index].name.toString(),
              currentMonthPrice:
                  "${MyHomePageState.buiFormatter.buiNumberFormatConversionWithCountryCode(dashboard[index].currentMonth, "In")}",
              lastMonthPrice:
                  "${MyHomePageState.buiFormatter.buiNumberFormatConversionWithCountryCode(dashboard[index].lastMonth, "in")}",
              trandValue:
                  dashboard[index].currentMonth >= dashboard[index].lastMonth
                      ? true
                      : false,
              cardDate: "(" +
                  "${MyHomePageState.buiFormatter.getPreviousMonthNameAndYear(now.month)}" +
                  ")",
              cardType: CardType.cardDashBoardDeal,
              cardSize: CardSize.small,
              width: MediaQuery.of(context).size.width,
              height: MediaQuery.of(context).size.height * 0.1,
              padding: 5,
              elevation: 30,
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(5),
              ),
            );
          }),
        ));
  }

  getGridViewSelectedItem(BuildContext context, String gridItem) {
    showDialog(
      context: context,
      builder: (BuildContext context) {
        return AlertDialog(
          title: new Text(gridItem),
          actions: <Widget>[
            FlatButton(
              child: new Text("OK"),
              onPressed: () {
                Navigator.of(context).pop();
              },
            ),
          ],
        );
      },
    );
  }

  Future<void> onMenuPressed() async {
    setState(() {
      widget.onMenuPressed();
      widget.onMenuIconPressed();
    });
  }

  Future<void> changeOrderView() async {
    setState(() {
      isGridView = !isGridView;
    });

    changeMode();
  }

  changeMode() {
    if (countValue == 2) {
      setState(() {
        countValue = 1;
        aspectWidth = 3;
        aspectHeight = 1;
      });
    } else {
      setState(() {
        countValue = 2;
        aspectWidth = 2;
        aspectHeight = 1;
      });
    }
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColor = prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }

    popUpMenu = [
      PopupMenuItem(
        height: 30,
        value: 1,
        child: Row(
          children: [
            Padding(
              padding: const EdgeInsets.all(12.0),
              child: Icon(
                Icons.settings,
                color: theme ?? AppColors.APP_COLOR_BLACK,
              ),
            ),
            Text(
              AppStrings.SETTINGS,
              style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_14,
                  color: theme ?? Colors.black87),
            )
          ],
        ),
      ),
      PopupMenuItem(
        value: 2,
        child: Row(
          children: [
            Padding(
              padding: const EdgeInsets.all(12.0),
              child: Icon(
                Icons.notifications_active,
                color: theme ?? AppColors.APP_COLOR_BLACK,
              ),
            ),
            Text(
              AppStrings.NOTIFICATION,
              style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_14,
                  color: theme ?? Colors.black87),
            )
          ],
        ),
      ),
      PopupMenuItem(
        value: 3,
        child: Row(
          children: [
            Padding(
              padding: const EdgeInsets.all(12.0),
              child: Icon(
                Icons.messenger_rounded,
                color: theme ?? AppColors.APP_COLOR_BLACK,
              ),
            ),
            Text(
              AppStrings.MESSAGE,
              style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_14,
                  color: theme ?? Colors.black87),
            )
          ],
        ),
      ),
      PopupMenuItem(
        value: 4,
        child: Row(
          children: [
            Padding(
              padding: const EdgeInsets.all(12.0),
              child: Icon(
                Icons.color_lens_sharp,
                color: theme ?? AppColors.APP_COLOR_BLACK,
              ),
            ),
            Text(
              AppStrings.THEME,
              style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_14,
                  color: theme ?? Colors.black87),
            )
          ],
        ),
      ),
      PopupMenuItem(
        value: 5,
        child: Row(
          children: [
            Padding(
              padding: const EdgeInsets.all(12.0),
              child: Icon(
                Icons.feedback_rounded,
                color: theme ?? AppColors.APP_COLOR_BLACK,
              ),
            ),
            Text(
              AppStrings.FEED_BACK,
              style: TextStyle(
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_14,
                  color: theme ?? Colors.black87),
            )
          ],
        ),
      ),
    ];

    if (secondaryColor != null) {
      int colorCodeInt = int.parse(secondaryColor);
      setState(() {
        secondaryBackgroundColor = Color(colorCodeInt);
        this.mounted;
      });

      return stringValue;
    } else {
      secondaryBackgroundColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  void onBackPressed() {
    colorCodePref();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Message Pressed");
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {});
  }

  Future<void> onSettingFunction() async {
    setState(() {
      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) => ButtonView(
                    primaryColor: theme,
                    secondaryColor: secondaryBackgroundColor,
                  )));

      Utils.showSnackBar(context, "Setting Pressed");
    });
  }

  Future<void> onNotificationFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Notification Pressed");
      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) => NoInternetScreen()));
    });
  }

  Future<void> onPopUpMenuPressed(int count) async {
    switch (count) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (BuildContext context) => ThemePickerScreen(
                  onBackPressed: onBackPressed,
                )));
  }

  Future<void> callDashBoardAPI() async {
    setState(() {
      isLoader = true;
      isAppUpdate = false;
    });
    String urlController = ServerUrl.CRM_DASHBOARD_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    DashBoardDetailBean dashBoardDetailBean = DashBoardDetailBean.fromJson(map);

    if (dashBoardDetailBean.status) {
      dashBoardDetailsBeanResponse = dashBoardDetailBean;
      dashboard = dashBoardDetailBean.data.dashboard;
      setState(() {
        isLoader = false;
        isAppUpdate = false;
      });
      Utils.showSnackBar(context, dashBoardDetailBean.message);
    }

    if (dashBoardDetailBean.statusCode == 401) {
      if (dashBoardDetailBean.message == AppStrings.UPDATE_APP) {
        setState(() {
          isLoader = false;
          isAppUpdate = true;
        });
      }
    }
  }
}
