import 'dart:convert';

import 'package:bui_component/ButtonsComponent/buttonComponent.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/contact_details_bean.dart';
import 'package:example/Models/deals_details_bean.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Models/leads_details_bean.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/Utils/loader.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

class ConditionalDropDown extends StatefulWidget {
  Color theme;
  Color secondaryColor;
  Function() onCallBack;

  ConditionalDropDown({
    Key key,
    this.theme,
    this.secondaryColor,
    this.onCallBack,
  }) : super(key: key);

  @override
  _ConditionalDropDownState createState() => _ConditionalDropDownState();
}

class _ConditionalDropDownState extends State<ConditionalDropDown> {
  String contactIndex = "";
  String accountIndex;
  String leadIndex = "";
  String dealIndex = "";
  String activityTypeIndex = "";
  String statusIndex = "";

  bool selected = false;

  bool isLoader = true;

  List<dynamic> account = new List();
  List<dynamic> status = new List();
  List<dynamic> contact = new List();
  List<dynamic> lead = new List();
  List<dynamic> activityType = new List();
  List<dynamic> deal = new List();

  final descController = TextEditingController();
  final dateController = TextEditingController();

  final _formKey = GlobalKey<FormState>();

  DateTime selectedDate = DateTime.now();
  String selectedDateString;
  String ownerId;

  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    initSharedPref();

    dateController.text =
        MyHomePageState.buiFormatter.dateToStringDate(selectedDate);

    callContactAPI();
  }

  Future<void> initSharedPref() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);
    setState(() {
      ownerId = ownerIdValue;
    });
  }

  @override
  void dispose() {
    descController.dispose();
    dateController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        key: _scaffoldKey,
        appBar: AppBar(
          title: Text("Create Task"),
          backgroundColor: widget.theme,
        ),
        body: isLoader
            ? Loader(
                loaderColor: widget.theme,
              )
            : _dependedDD);
  }

  Widget get _dependedDD {
    return Form(
      key: _formKey,
      child: ListView(
        //mainAxisAlignment: MainAxisAlignment.start,
        children: [
          Container(
            padding: const EdgeInsets.all(5.0),
            child: RichText(
              text: TextSpan(
                  text: "Contact",
                  style: TextStyle(
                      color: AppColors.appBlack,
                      fontWeight: FontWeight.bold,
                      fontSize: FontSize.FONT_16),
                  children: [
                    TextSpan(
                        text: ' *',
                        style: TextStyle(
                            color: Colors.red,
                            fontWeight: FontWeight.bold,
                            fontSize: FontSize.FONT_16))
                  ]),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: Container(
              width: MediaQuery.of(context).size.width,
              padding: const EdgeInsets.only(left: 8.0, right: 8.0),
              //  height: selected ? 10 : 20,
              decoration: ShapeDecoration(
                color: Colors.white,
                shape: RoundedRectangleBorder(
                  side: BorderSide(
                      width: 1.0,
                      style: BorderStyle.solid,
                      color: widget.theme),
                  borderRadius: BorderRadius.all(
                    Radius.circular(5.0),
                  ),
                ),
              ),
              child: DropdownButtonFormField(
                validator: (value) {
                  if (value.length == 0 && value.toString().isEmpty) {
                    return "Please Select Any Value";
                  }
                  return null;
                },
                menuMaxHeight: 300,
                isExpanded: true,
                icon: const Icon(
                  Icons.arrow_drop_down_outlined,
                  size: 20,
                ),
                items: contact.map((item) {
                  return new DropdownMenuItem(
                    child: Container(child: new Text(item['label'])),
                    value: item['value'].toString(),
                  );
                }).toList(),
                onChanged: (newVal) {
                  setState(() {
                    contactIndex = newVal;

                    if (contactIndex != '') {
                      callContactFilterAPI(contactIndex);
                    }
                  });
                },
                value: contactIndex,
                hint: Text(
                  'Select a province',
                ),
              ),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: RichText(
              text: TextSpan(
                  text: "Account ID",
                  style: TextStyle(
                      color: AppColors.appBlack,
                      fontWeight: FontWeight.bold,
                      fontSize: FontSize.FONT_16),
                  children: [
                    TextSpan(
                        text: ' *',
                        style: TextStyle(
                            color: Colors.red,
                            fontWeight: FontWeight.bold,
                            fontSize: FontSize.FONT_16))
                  ]),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: Container(
              width: MediaQuery.of(context).size.width,
              padding: const EdgeInsets.only(left: 8.0, right: 8.0),
              //  height: selected ? 10 : 20,
              decoration: ShapeDecoration(
                color: Colors.white,
                shape: RoundedRectangleBorder(
                  side: BorderSide(
                      width: 1.0,
                      style: BorderStyle.solid,
                      color: widget.theme),
                  borderRadius: BorderRadius.all(
                    Radius.circular(5.0),
                  ),
                ),
              ),
              child: DropdownButtonFormField(
                menuMaxHeight: 300,
                isExpanded: true,
                validator: (value) {
                  if (value.toString().isEmpty) {
                    return "Please Select a Contact";
                  }
                  return null;
                },
                icon: const Icon(
                  Icons.arrow_drop_down_outlined,
                  size: 20,
                ),
                items: account.map((item) {
                      return new DropdownMenuItem(
                        child: Container(child: new Text(item['label'])),
                        value: item['value'].toString(),
                      );
                    }).toList() ??
                    [],
                onChanged: (newVal) {
                  setState(() {
                    accountIndex = newVal;
                  });
                },
                value: accountIndex,
                hint: Text('Select Account ID'),
              ),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: RichText(
              text: TextSpan(
                  text: "Description",
                  style: TextStyle(
                      color: AppColors.appBlack,
                      fontWeight: FontWeight.bold,
                      fontSize: FontSize.FONT_16),
                  children: [
                    TextSpan(
                        text: ' *',
                        style: TextStyle(
                            color: Colors.red,
                            fontWeight: FontWeight.bold,
                            fontSize: FontSize.FONT_16))
                  ]),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: TextFormField(
              controller: descController,
              decoration: new InputDecoration(
                filled: true,
                fillColor: Colors.white,
                hintText: "Description",
                enabledBorder: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(5.0),
                  borderSide: BorderSide(
                    color: widget.theme,
                    width: 1.0,
                  ),
                ),
                focusedBorder: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(5.0),
                  borderSide: BorderSide(
                    color: widget.theme,
                  ),
                ),
                contentPadding: const EdgeInsets.all(5.0),
                border: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(5.0),
                ),
              ),
              validator: (value) {
                if (value.length == 0) {
                  return "Please Enter Some Text";
                }
                return null;
              },
              onChanged: (String value) {},
              onFieldSubmitted: (value) {
                Utils.hideKeyboard(context);
              },
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: RichText(
              text: TextSpan(
                  text: "Deal",
                  style: TextStyle(
                      color: AppColors.appBlack,
                      fontWeight: FontWeight.bold,
                      fontSize: FontSize.FONT_16),
                  children: [
                    TextSpan(
                        text: ' *',
                        style: TextStyle(
                            color: Colors.red,
                            fontWeight: FontWeight.bold,
                            fontSize: FontSize.FONT_16))
                  ]),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: Container(
              width: MediaQuery.of(context).size.width,
              padding: const EdgeInsets.only(left: 8.0, right: 8.0),
              //  height: selected ? 10 : 20,
              decoration: ShapeDecoration(
                color: Colors.white,
                shape: RoundedRectangleBorder(
                  side: BorderSide(
                      width: 1.0,
                      style: BorderStyle.solid,
                      color: widget.theme),
                  borderRadius: BorderRadius.all(
                    Radius.circular(5.0),
                  ),
                ),
              ),
              child: DropdownButtonFormField(
                menuMaxHeight: 300,
                isExpanded: true,
                validator: (value) {
                  if (value.toString().isEmpty) {
                    return "Please Select a Contact";
                  }
                  return null;
                },
                icon: const Icon(
                  Icons.arrow_drop_down_outlined,
                  size: 20,
                ),
                items: account.map((item) {
                      return new DropdownMenuItem(
                        child: Container(child: new Text(item['label'])),
                        value: item['value'].toString(),
                      );
                    }).toList() ??
                    [],
                onChanged: (newVal) {
                  setState(() {
                    accountIndex = newVal;
                  });
                },
                value: accountIndex,
                hint: Text('Select a Deal'),
              ),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: RichText(
              text: TextSpan(
                  text: "Lead",
                  style: TextStyle(
                      color: AppColors.appBlack,
                      fontWeight: FontWeight.bold,
                      fontSize: FontSize.FONT_16),
                  children: [
                    TextSpan(
                        text: ' *',
                        style: TextStyle(
                            color: Colors.red,
                            fontWeight: FontWeight.bold,
                            fontSize: FontSize.FONT_16))
                  ]),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: Container(
              width: MediaQuery.of(context).size.width,
              padding: const EdgeInsets.only(left: 8.0, right: 8.0),
              //  height: selected ? 10 : 20,
              decoration: ShapeDecoration(
                color: Colors.white,
                shape: RoundedRectangleBorder(
                  side: BorderSide(
                      width: 1.0,
                      style: BorderStyle.solid,
                      color: widget.theme),
                  borderRadius: BorderRadius.all(
                    Radius.circular(5.0),
                  ),
                ),
              ),
              child: DropdownButtonFormField(
                menuMaxHeight: 300,
                isExpanded: true,
                validator: (value) {
                  if (value.toString().isEmpty) {
                    return "Please Select Any Lead";
                  }
                  return null;
                },
                icon: const Icon(
                  Icons.arrow_drop_down_outlined,
                  size: 20,
                ),
                items: lead.map((item) {
                      return new DropdownMenuItem(
                        child: Container(child: new Text(item['label'])),
                        value: item['value'].toString(),
                      );
                    }).toList() ??
                    [],
                onChanged: (newVal) {
                  setState(() {
                    leadIndex = newVal;
                  });
                },
                value: leadIndex,
                hint: Text('Select a Lead'),
              ),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: RichText(
              text: TextSpan(
                  text: "Activity Type",
                  style: TextStyle(
                      color: AppColors.appBlack,
                      fontWeight: FontWeight.bold,
                      fontSize: FontSize.FONT_16),
                  children: [
                    TextSpan(
                        text: ' *',
                        style: TextStyle(
                            color: Colors.red,
                            fontWeight: FontWeight.bold,
                            fontSize: FontSize.FONT_16))
                  ]),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: Container(
              width: MediaQuery.of(context).size.width,
              padding: const EdgeInsets.only(left: 8.0, right: 8.0),
              //  height: selected ? 10 : 20,
              decoration: ShapeDecoration(
                color: Colors.white,
                shape: RoundedRectangleBorder(
                  side: BorderSide(
                      width: 1.0,
                      style: BorderStyle.solid,
                      color: widget.theme),
                  borderRadius: BorderRadius.all(
                    Radius.circular(5.0),
                  ),
                ),
              ),
              child: DropdownButtonFormField(
                menuMaxHeight: 300,
                isExpanded: true,
                validator: (value) {
                  if (value.toString().isEmpty) {
                    return "Please Select Any Activity Type";
                  }
                  return null;
                },
                icon: const Icon(
                  Icons.arrow_drop_down_outlined,
                  size: 20,
                ),
                items: activityType.map((item) {
                      return new DropdownMenuItem(
                        child: Container(child: new Text(item['label'])),
                        value: item['value'].toString(),
                      );
                    }).toList() ??
                    [],
                onChanged: (newVal) {
                  setState(() {
                    activityTypeIndex = newVal;
                  });
                },
                value: activityTypeIndex,
                hint: Text('Select a Activity Type'),
              ),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: RichText(
              text: TextSpan(
                  text: "Due Date",
                  style: TextStyle(
                      color: AppColors.appBlack,
                      fontWeight: FontWeight.bold,
                      fontSize: FontSize.FONT_16),
                  children: [
                    TextSpan(
                        text: ' *',
                        style: TextStyle(
                            color: Colors.red,
                            fontWeight: FontWeight.bold,
                            fontSize: FontSize.FONT_16))
                  ]),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: TextFormField(
              controller: dateController,
              onTap: () {
                // Below line stops keyboard from appearing
                Utils.hideKeyboard(context);
                _selectDate(context);
                // Show Date Picker Here
              },
              decoration: new InputDecoration(
                filled: true,
                fillColor: Colors.white,
                hintText: "Description",
                enabledBorder: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(5.0),
                  borderSide: BorderSide(
                    color: widget.theme,
                    width: 1.0,
                  ),
                ),
                focusedBorder: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(5.0),
                  borderSide: BorderSide(
                    color: widget.theme,
                  ),
                ),
                contentPadding: const EdgeInsets.all(5.0),
                border: OutlineInputBorder(
                  borderRadius: BorderRadius.circular(5.0),
                ),
              ),
              validator: (value) {
                if (value.length == 0) {
                  return "Please Enter Some Text";
                }
                return null;
              },
              onChanged: (String value) {},
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: RichText(
              text: TextSpan(
                  text: "Status",
                  style: TextStyle(
                      color: AppColors.appBlack,
                      fontWeight: FontWeight.bold,
                      fontSize: FontSize.FONT_16),
                  children: [
                    TextSpan(
                        text: ' *',
                        style: TextStyle(
                            color: Colors.red,
                            fontWeight: FontWeight.bold,
                            fontSize: FontSize.FONT_16))
                  ]),
            ),
          ),
          Container(
            padding: const EdgeInsets.all(5.0),
            child: Container(
              width: MediaQuery.of(context).size.width,
              padding: const EdgeInsets.only(left: 8.0, right: 8.0),
              //  height: selected ? 10 : 20,
              decoration: ShapeDecoration(
                color: Colors.white,
                shape: RoundedRectangleBorder(
                  side: BorderSide(
                      width: 1.0,
                      style: BorderStyle.solid,
                      color: widget.theme),
                  borderRadius: BorderRadius.all(
                    Radius.circular(5.0),
                  ),
                ),
              ),
              child: DropdownButtonFormField(
                menuMaxHeight: 300,
                isExpanded: true,
                validator: (value) {
                  if (value.toString().isEmpty) {
                    return "Please Select Any Status";
                  }
                  return null;
                },
                icon: const Icon(
                  Icons.arrow_drop_down_outlined,
                  size: 20,
                ),
                items: status.map((item) {
                      return new DropdownMenuItem(
                        child: Container(child: new Text(item['label'])),
                        value: item['value'].toString(),
                      );
                    }).toList() ??
                    [],
                onChanged: (newVal) {
                  setState(() {
                    statusIndex = newVal;
                  });
                },
                value: statusIndex,
                hint: Text('Select a Status'),
              ),
            ),
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: [
              Container(
                padding: const EdgeInsets.all(5.0),
                child: Button.mini(
                  buttonType: ButtonType.custom,
                  btnTextColor: widget.secondaryColor,
                  btnText: "Cancel",
                  btnColor: Colors.white,
                  width: 120,
                  buttonSize: ButtonSize.medium,
                  elevation: 20,
                  onPressed: () {
                    Navigator.of(context).pop();
                  },
                ),
              ),
              Container(
                padding: const EdgeInsets.all(5.0),
                child: Button.mini(
                  buttonType: ButtonType.custom,
                  btnColor: widget.secondaryColor,
                  btnText: "Save",
                  btnTextColor: Colors.white,
                  width: 120,
                  buttonSize: ButtonSize.medium,
                  elevation: 20,
                  onPressed: () {
                    final isValid = _formKey.currentState.validate();
                    if (!isValid) {
                      return;
                    } else {
                      callTaskAddedAPI();
                    }
                    _formKey.currentState.save();
                  },
                ),
              ),
            ],
          ),
        ],
      ),
    );
  }

  Future<void> _selectDate(BuildContext context) async {
    final DateTime picked = await showDatePicker(
        context: context,
        initialDate: selectedDate,
        firstDate: DateTime(2000, 8),
        lastDate: DateTime(2101));
    if (picked != null && picked != selectedDate)
      setState(() {
        selectedDate = picked;
        selectedDateString =
            MyHomePageState.buiFormatter.dateToStringDate(selectedDate);
        dateController.text = selectedDateString;
      });
  }

  Future<void> callContactFilterAPI([String province]) async {
    account.clear();
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_CONTACT_URL;
    var objToSen;

    objToSen = {
      "\"condition\"": [
        {"\"id\"": province.toString()}
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSen.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    ContactDetailsBean contactDetailsResponseBean =
        ContactDetailsBean.fromJson(map);

    if (contactDetailsResponseBean.status) {
      if (province != '') {
        account = [
          for (int i = 0; i < contactDetailsResponseBean.data.length; i++)
            if (contactDetailsResponseBean.data[i].accountid != null)
              {
                'label': contactDetailsResponseBean
                    .data[i].accountid.companyname
                    .toString(),
                'value':
                    contactDetailsResponseBean.data[i].accountid.id.toString(),
              },
        ];

        final jsonList1 = account.map((item) => jsonEncode(item)).toList();

        final uniqueJsonList1 = jsonList1.toSet().toList();

        account = uniqueJsonList1.map((item) => jsonDecode(item)).toList();

        isLoader = true;
      }

      accountIndex = account[0]['value'].toString();

      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> callContactAPI() async {
    contact.clear();
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_CONTACT_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    ContactDetailsBean contactDetailsResponseBean =
        ContactDetailsBean.fromJson(map);

    if (contactDetailsResponseBean.status) {
      contact = [
        for (int i = 0; i < contactDetailsResponseBean.data.length; i++)
          {
            'label': contactDetailsResponseBean.data[i].firstname +
                " " +
                contactDetailsResponseBean.data[i].secondName,
            'value': contactDetailsResponseBean.data[i].id.toString(),
          },
      ];

      contact.insert(0, {
        'label': '',
        'value': '',
      });

      leadScreenViewDetailsAPI();
    }
  }

  Future<void> leadScreenViewDetailsAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_LEADS_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    LeadsDetailsBean leadDetailsResponseBean = LeadsDetailsBean.fromJson(map);

    if (leadDetailsResponseBean.status) {
      lead = [
        for (int i = 0; i < leadDetailsResponseBean.data.length; i++)
          {
            'label': leadDetailsResponseBean.data[i].firstname +
                " " +
                leadDetailsResponseBean.data[i].lastname,
            'value': leadDetailsResponseBean.data[i].id.toString(),
          },
      ];
      setState(() {
        isLoader = false;
      });
      lead.insert(
        0,
        {
          'label': '',
          'value': '',
        },
      );
      dealScreenViewDetailsAPI();
    }
  }

  Future<void> dealScreenViewDetailsAPI({String obj}) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_DEALS_URL;
    var objToSend;
    if (obj == null) {
      objToSend = {
        "\"condition\"": [],
        "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
        "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
        "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
      };
    }
    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    DealsDetailsBean dealsDetailsBean = DealsDetailsBean.fromJson(map);

    if (dealsDetailsBean.status) {
      deal = [
        for (int i = 0; i < dealsDetailsBean.data.length; i++)
          {
            'label': dealsDetailsBean.data[i].companyid.companyname.toString(),
            'value': dealsDetailsBean.data[i].id.toString(),
          },
      ];

      deal.insert(
        0,
        {
          'label': '',
          'value': '',
        },
      );
      setState(() {
        isLoader = false;
      });
      taskMasterViewAPI();
    }
  }

  Future<void> taskMasterViewAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_MASTER_VIEW;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);

    if (detailsBean.statusCode == 400) {}
    if (detailsBean.status) {
      activityType = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Activity")
            if (detailsBean.data[i].key == "Type")
              {
                'label': detailsBean.data[i].name,
                'value': detailsBean.data[i].id.toString(),
              },
      ];
      status = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Activity")
            if (detailsBean.data[i].key == "Status")
              {
                'label': detailsBean.data[i].name,
                'value': detailsBean.data[i].id.toString(),
              },
      ];

      activityType.insert(
        0,
        {
          'label': '',
          'value': '',
        },
      );

      status.insert(
        0,
        {
          'label': '',
          'value': '',
        },
      );

      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> callTaskAddedAPI() async {
    var description = descController.text;
    var dateReverse =
        MyHomePageState.buiFormatter.dateToStringDateReverse(selectedDate);
    var objToSend = {
      "\"owner_id\"": "\"$ownerId\"",
      "\"accountid\"": "\"$accountIndex\"",
      "\"contactid\"": "\"$contactIndex\"",
      "\"desc\"": "\"$description\"",
      "\"leadid\"": "\"$leadIndex\"",
      "\"activitytypeid\"": "\"$activityTypeIndex\"",
      "\"dealid\"": "\"$accountIndex\"",
      "\"duedate\"": "\"$dateReverse\"",
      "\"statusid\"": "\"$statusIndex\"",
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String urlController = ServerUrl.CRM_CREATE_TASK_URL;
    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: objToSend.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      Utils.showInSnackBar(
          scaffoldKey: _scaffoldKey, message: baseBean.message);
      widget.onCallBack();
      Navigator.pop(context);
    } else {
      Utils.showInSnackBar(
          scaffoldKey: _scaffoldKey, message: baseBean.message);
    }
  }
}
