import 'dart:convert';

import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/SvgPictures.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:bui_component/CustomAlertComponent/CustomeAlert.dart';
import 'package:bui_component/DataModalComponent/SlidableListView.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/contact_details_bean.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Utils/DrawerMenuDesign.dart';
import 'package:example/Utils/loader.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';

import '../ButtonView.dart';
import '../CardStyle.dart';
import '../ThemePickerScreen.dart';
import 'FilterScreen.dart';
import 'appUpdate.dart';

class ContactPage extends DrawerContent {
  ContactPage({Key key, this.title});

  final String title;

  @override
  _ContactPageState createState() => _ContactPageState();
}

class _ContactPageState extends State<ContactPage> {
  Color theme = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
  SharedPreferences prefs;

  ContactDetailsBean contactDetailsBean;

  List<ContactListBean> contactListBeanData = new List();

  bool isLoader = true;

  List<String> actions = ["Edit", "Delete"];

  String jsonFormFilterData;

  List<dynamic> status = new List();
  List<dynamic> industry = new List();

  @override
  void initState() {
    // TODO: implement initState

    callPostAPI();
    getStringValuesSF();

    super.initState();
  }

  getStringValuesSF() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);
      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }
    String secondaryColor = prefs.getString(AppStrings.SECONDARY_THEME_COLOR);
    if (secondaryColor != null) {
      int colorCodeInt = int.parse(secondaryColor);
      setState(() {
        secondaryColorTheme = Color(colorCodeInt);
        this.mounted;
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
        secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
        this.mounted;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: isLoader
          ? Loader(
              loaderColor: Colors.red,
            )
          : Center(
              child: Column(
                children: <Widget>[
                  /* Container(
                    color: theme ?? AppColors.primaryColor,
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: <Widget>[
                        ClipRRect(
                          borderRadius: BorderRadius.all(Radius.circular(32.0)),
                          child: Material(
                            shadowColor: Colors.transparent,
                            color: Colors.transparent,
                            child: IconButton(
                              icon: Icon(
                                Icons.menu,
                                color: Colors.white,
                              ),
                              onPressed: () {
                                widget.onMenuPressed();
                              },
                            ),
                          ),
                        ),
                        Row(
                          children: [
                            Visibility(
                              visible: true,
                              child: InkWell(
                                onTap: () {
                                  onFilterClicked();
                                },
                                child: Container(
                                  //padding: EdgeInsets.only(right: 2),
                                  child: Icon(
                                    Icons.filter_alt,
                                    color: AppColors.appWhite,
                                  ),
                                ),
                              ),
                            ),
                            Container(
                                //padding: EdgeInsets.only(right: 2),
                                child: PopupMenuButton(
                              elevation: 30,
                              //offset: Offset(0, 100),
                              tooltip: "More Feature",
                              itemBuilder: (context) => [
                                PopupMenuItem(
                                  height: 30,
                                  value: 1,
                                  child: Row(
                                    children: [
                                      Padding(
                                        padding: const EdgeInsets.all(12.0),
                                        child: Icon(
                                          Icons.settings,
                                          color: theme ??
                                              AppColors.APP_COLOR_BLACK,
                                        ),
                                      ),
                                      Text(
                                        AppStrings.SETTINGS,
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            fontSize: FontSize.FONT_14,
                                            color: theme ?? Colors.black87),
                                      )
                                    ],
                                  ),
                                ),
                                PopupMenuItem(
                                  value: 2,
                                  child: Row(
                                    children: [
                                      Padding(
                                        padding: const EdgeInsets.all(12.0),
                                        child: Icon(
                                          Icons.notifications_active,
                                          color: theme ??
                                              AppColors.APP_COLOR_BLACK,
                                        ),
                                      ),
                                      Text(
                                        AppStrings.NOTIFICATION,
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            fontSize: FontSize.FONT_14,
                                            color: theme ?? Colors.black87),
                                      )
                                    ],
                                  ),
                                ),
                                PopupMenuItem(
                                  value: 3,
                                  child: Row(
                                    children: [
                                      Padding(
                                        padding: const EdgeInsets.all(12.0),
                                        child: Icon(
                                          Icons.messenger_rounded,
                                          color: theme ??
                                              AppColors.APP_COLOR_BLACK,
                                        ),
                                      ),
                                      Text(
                                        AppStrings.MESSAGE,
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            fontSize: FontSize.FONT_14,
                                            color: theme ?? Colors.black87),
                                      )
                                    ],
                                  ),
                                ),
                                PopupMenuItem(
                                  value: 4,
                                  child: Row(
                                    children: [
                                      Padding(
                                        padding: const EdgeInsets.all(12.0),
                                        child: Icon(
                                          Icons.color_lens_sharp,
                                          color: theme ??
                                              AppColors.APP_COLOR_BLACK,
                                        ),
                                      ),
                                      Text(
                                        AppStrings.THEME,
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            fontSize: FontSize.FONT_14,
                                            color: theme ?? Colors.black87),
                                      )
                                    ],
                                  ),
                                ),
                                PopupMenuItem(
                                  value: 5,
                                  child: Row(
                                    children: [
                                      Padding(
                                        padding: const EdgeInsets.all(12.0),
                                        child: Icon(
                                          Icons.feedback_rounded,
                                          color: theme ??
                                              AppColors.APP_COLOR_BLACK,
                                        ),
                                      ),
                                      Text(
                                        AppStrings.FEED_BACK,
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            fontSize: FontSize.FONT_14,
                                            color: theme ?? Colors.black87),
                                      )
                                    ],
                                  ),
                                ),
                              ],
                              onCanceled: () {},
                              onSelected: (value) {
                                onPopUpMenuFunction(value);
                              },
                              icon: Icon(
                                Icons.more_vert_sharp,
                                color: AppColors.appWhite,
                              ),
                            )),
                          ],
                        ),
                      ],
                    ),
                  ),*/
                  DrawerMenuDesign(
                    onMenuClicked: widget.onMenuPressed,
                    isFilterEnabled: true,
                    isFilterClicked: onFilterClicked,
                  ),
                  Container(child: detailsView),
                ],
              ),
            ),
    );
  }

  Future<void> onPopUpMenuFunction(int s) async {
    switch (s) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;
      case 6:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (BuildContext context) => ThemePickerScreen(
                  onBackPressed: onBackPressed,
                )));
  }

  void onBackPressed() {
    getStringValuesSF();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Toast.show("Message Pressed", context);
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {
      Navigator.push(context,
          MaterialPageRoute(builder: (BuildContext context) => CardStyle()));
    });
  }

  Future<void> onSettingFunction() async {
    setState(() {
      Toast.show("Setting Pressed", context);

      Navigator.push(context,
          MaterialPageRoute(builder: (BuildContext context) => ButtonView()));
    });
  }

  Future<void> onFilterClicked() async {
    setState(() {
      /*   jsonFormFilterData = json.encode({
        'fields': [
          {
            'apiKeyValue': 'startDate',
            'key': 'date',
            'type': 'date',
            'label': 'Start Date',
            'required': false,
          },
          {
            'apiKeyValue': 'endDate',
            'key': 'date',
            'type': 'date',
            'label': 'End Date'
          },


          {
            'apiKeyValue': 'industry',
            'key': 'Industry',
            'type': 'select',
            'label': 'Industry',
            'required': true,
            'items': industry != null
                ? industry
                : [
              {'label': '1', 'value': '2'}
            ]
          },

           {
            'apiKeyValue': 'statusid',
            'key': 'Status',
            'type': 'select',
            'label': 'Status',
            'required': true,
            'value': '1',
            'items': status != null
                ? status
                : [
              {'label': 'New', 'value': '1'},
              {'label': 'Active', 'value': '2'},
              {'label': 'Inactive', 'value': '3'},
            ]
          },
              {
            'apiKeyValue': 'industry',
            'key': 'select1',
            'type': 'select',
            'label': 'Industry',
            'required': false,
            'value': '',
            'items': [
              {
                'label': "Select",
                'value': "",
              },
              {"label": "Software", "value": "41"},
              {"label": "Hardware", "value": "42"},
              {"label": "Consumar goods", "value": "43"},
              {"label": "Farm & Farm Equipments", "value": "44"},
              {"label": "Manufacturing", "value": "45"},
              {"label": "Foundry", "value": "46"},
              {"label": "Automobile", "value": "47"},
              {"label": "Textile", "value": "48"},
              {"label": "Others", "value": "49"},
            ],
          },
          {
            'apiKeyValue': 'statusid',
            'key': 'select1',
            'type': 'select',
            'label': 'Status',
            'required': false,
            'items': [
              {"label": "New", "value": "6"},
              {"label": "Active", "value": "7"},
              {"label": "Inactive", "value": "8"},
              {'label': "Select", 'value': ""}
            ],
            "value": ""
          },
        ]
      });*/
      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) => FilterScreen(
                    primaryColor: theme,
                    secondaryColor: secondaryColorTheme,
                    filterFormData: jsonFormFilterData,
                    fromPageId: "accountId",
                    isActionApply: (String obj) {
                      callFilterChangesAPI(obj);
                    },
                  )));
    });
  }

  Future<void> onNotificationFunction() async {
    setState(() {
      Toast.show("Notification Pressed", context);

      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) => AppForceUpdate()));
    });
  }

  Future<void> callFilterChangesAPI(String obj) async {
    callPostAPI(obj: obj);
  }

  Future<void> accountMastViewAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_MASTER_VIEW;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);

    if (detailsBean.statusCode == 401) {}
    if (detailsBean.status) {
      status = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Account")
            if (detailsBean.data[i].key == "Status")
              {
                "'label'": json.encode(detailsBean.data[i].name),
                "'value'": json.encode(detailsBean.data[i].id.toString()),
              },
      ];
      industry = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].key == "Industry")
            {
              json.encode('label'): json.encode(detailsBean.data[i].name),
              json.encode('value'):
                  json.encode(detailsBean.data[i].id.toString()),
            },
      ];

      status
          .add({"'label'": json.encode("Select"), "'value'": json.encode("")});

      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Widget get detailsView {
    if (contactListBeanData.length != 0) {
      return Container(
        height: MediaQuery.of(context).size.height * 0.85,
        child: SlideListView(
          itemBuilder: (bc, index) {
            return InkWell(
              onTap: () {
                Toast.show(contactListBeanData[index].email, context);
              },
              child: BUICard.dashBoardCard(
                width: MediaQuery.of(context).size.width,
                cardSize: CardSize.large,
                cardTitle: contactListBeanData[index].firstname ?? "",
                cardName:
                    contactListBeanData[index].accountid.companyname ?? "",
                statusTag: contactListBeanData[index].statusid.name ?? "",
                dashBoardCardTime:
                    contactListBeanData[index].createdAt.substring(0, 10) ?? "",
                height: MediaQuery.of(context).size.height * 0.24,
                elevation: 10,
                isTagValue: false,
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(5.0),
                ),
                //tagValue: ["#one"],
              ),
            );
            /* return Container(
              width: MediaQuery.of(context).size.width,
              padding: EdgeInsets.all(5),
              child: Card(
                elevation: 5,
                shape: BeveledRectangleBorder(
                  borderRadius: BorderRadius.only(
                    bottomLeft: Radius.circular(15.0),
                    bottomRight: Radius.circular(15.0),
                    topLeft: Radius.circular(15.0),
                    topRight: Radius.circular(15.0),
                  ),
                ),
                child: Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Container(
                        width: MediaQuery.of(context).size.width,
                        decoration: BoxDecoration(
                            color: Colors.white,
                            shape: BoxShape.rectangle,
                            borderRadius: BorderRadius.only(
                                topLeft: Radius.circular(15.0),
                                topRight: Radius.circular(15.0))),
                        padding: EdgeInsets.only(left: 20, right: 10, top: 15),
                        child: Text(
                          '${contactListBeanData[index].accountid.companyname.toString()}',
                          overflow: TextOverflow.ellipsis,
                          style: TextStyle(
                              fontWeight: FontWeight.bold,
                              fontSize: FontSize.FONT_20,
                              letterSpacing: 0.5,
                              color: AppColors.APP_COLOR_BLACK),
                        )),
                    Container(
                      padding: EdgeInsets.only(left: 20, right: 10, top: 10),
                      child: Text(
                        '${contactListBeanData[index].firstname}',
                        style: TextStyle(
                            fontWeight: FontWeight.normal,
                            fontFamily: FontFamily.Gilroy,
                            fontSize: FontSize.FONT_15,
                            letterSpacing: 1,
                            color: AppColors.APP_COLOR_BLACK),
                      ),
                    ),
                    Container(
                        padding: EdgeInsets.only(left: 20, right: 10, top: 10),
                        child: Text(
                          '${contactListBeanData[index].email.toString()}',
                          style: TextStyle(
                              fontWeight: FontWeight.normal,
                              fontFamily: FontFamily.Gilroy,
                              fontSize: FontSize.FONT_15,
                              letterSpacing: 1,
                              color: AppColors.APP_COLOR_BLACK),
                        )),
                    Container(
                      padding: EdgeInsets.only(right: 15, top: 10, bottom: 10),
                      decoration: BoxDecoration(
                          color: Colors.white,
                          shape: BoxShape.rectangle,
                          borderRadius: BorderRadius.only(
                              bottomLeft: Radius.circular(15.0),
                              bottomRight: Radius.circular(15.0))),
                      alignment: Alignment.bottomRight,
                      child: Button.mini(


                        elevation: 1,
                        width: MediaQuery.of(context).size.width * 0.2,
                        onPressed: () {
                          */ /*Navigator.of(context).push(MaterialPageRoute(
                            builder: (context) {
                              return ViewMoreScreen(
                                pageTitle: "Account Recent",
                                accountListBeanData: accountListBeanData[index],
                                theme: theme,
                              );
                            },
                          ));*/ /*
                        },
                        btnTextColor: AppColors.appWhite,
                        btnColor: theme ?? AppColors.APP_COLOR_PRIMARY,
                        btnText: AppStrings.VIEW_MORE,
                        buttonType: ButtonType.custom,
                      ),
                    ),
                  ],
                ),
              ));*/
          },
          actionWidgetDelegate:
              ActionWidgetDelegate(actions.length, (actionIndex, listIndex) {
            if (actionIndex == 0) {
              return Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Icon(
                    Icons.edit,
                    color: theme ?? AppColors.primaryColor,
                  ),
                  Text(
                    actions[0].toString(),
                    overflow: TextOverflow.ellipsis,
                    style: TextStyle(
                      fontWeight: FontWeight.bold,
                      fontSize: 14,
                      color: theme ?? AppColors.primaryColor,
                    ),
                  ),
                ],
              );
            } else {
              return Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Icon(Icons.delete, color: AppColors.errorWarning),
                  Text(
                    actions[1].toString(),
                    overflow: TextOverflow.ellipsis,
                    style: TextStyle(
                        fontWeight: FontWeight.bold,
                        fontSize: 12,
                        color: AppColors.errorWarning),
                  ),
                ],
              );
            }
          }, (int indexInList, int index, BaseSlideItem item) {
            if (index == 1) {
              //item.remove();

              CustomAlertBox.showCustomAlertBox(
                context: context,
                type: "Warning",
                enterWidget:
                    Text("Are you sure do you want to delete this Item"),
                isPositiveEnabled: true,
                isNegativeEnabled: true,
                setPositiveAction: () {
                  item.remove();
                  // Navigator.pop(context);
                },
              );
              //  _showMyDialog(indexInList, item);
            } else {
              item.close();
              CustomAlertBox.showCustomAlertBox(
                context: context,
                type: "Alert",
                enterWidget: Text("This is Sample Alert"),
                isPositiveEnabled: true,
                isNegativeEnabled: false,
                positiveButtonColor: Colors.green,
                setPositiveAction: () {},
              );
              Toast.show("Close this Item", context);
            }
          }, [Colors.white, Colors.white]),
          dataList: contactListBeanData,
          itemBackgroundColor: Colors.grey[200],
          refreshCallback: () async {
            await Future.delayed(Duration(seconds: 2));

            setState(() {
              // fetchData();

              callPostAPI();

              this.mounted;
            });
            return;
          },
        ),
      );

      /*   return Container(
        height: 200,
        child: ListView.builder(
            shrinkWrap: true,
            itemCount: accountListBeanData.length,
            itemBuilder: (BuildContext context, int index) {
              return Container(
                height: 50,
                color: Colors.amber,
                child: Card(
                  child: Text(
                      accountListBeanData[index].companyname.toString() ?? ""),
                ),
              );
            }),
      );*/
    } else {
      return Container(
          margin: EdgeInsets.only(top: 90),
          child: Center(
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                SvgPicture.asset(
                  SvgPictures.NO_DATA_FOUND,
                  placeholderBuilder: (context) => CircularProgressIndicator(),
                  height: 128.0,
                  allowDrawingOutsideViewBox: false,
                ),
                SizedBox(
                  height: 20,
                ),
                Text(
                  "OOPS, There is no data found",
                  style: TextStyle(
                      fontWeight: FontWeight.bold,
                      fontSize: FontSize.FONT_20,
                      color: AppColors.APP_GRAY_1),
                ),
              ],
            ),
          ));
    }
  }

  Future<void> callPostAPI({String obj}) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_CONTACT_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    ContactDetailsBean contactDetailsResponseBean =
        ContactDetailsBean.fromJson(map);

    if (contactDetailsResponseBean.status) {
      contactDetailsBean = contactDetailsResponseBean;
      contactListBeanData = contactDetailsResponseBean.data;

      setState(() {
        isLoader = false;
      });
      Toast.show(contactDetailsBean.message.toString(), context);
    }
  }
}
