import 'dart:convert';

import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/account_details_bean.dart';
import 'package:example/Models/dash_board_detail_bean.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Screen/ERP/ConditionalDropDown.dart';
import 'package:example/Utils/DrawerMenuDesign.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/Utils/loader.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:universal_platform/universal_platform.dart';

import '../../main.dart';
import '../ButtonView.dart';
import '../CardStyle.dart';
import '../ThemePickerScreen.dart';
import 'FilterScreen.dart';
import 'appUpdate.dart';

class DashboardPage extends DrawerContent {
  DashboardPage({Key key, this.title});

  final String title;

  @override
  _DashboardPageState createState() => _DashboardPageState();
}

class _DashboardPageState extends State<DashboardPage> {
  Color theme = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
  SharedPreferences prefs;

  num countValue = 2;

  num aspectWidth = 2;

  num aspectHeight = 1;

  List<Dashboard> dashboard = new List();

  bool isGridView = false;

  DashBoardDetailBean dashBoardDetailsBeanResponse;
  var now = DateTime.now();

  @override
  void initState() {
    // TODO: implement initState

    callDashBoardAPI();

    getStringValuesSF();

    super.initState();
  }

  getStringValuesSF() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);
      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }
    String secondaryColor = prefs.getString(AppStrings.SECONDARY_THEME_COLOR);
    if (secondaryColor != null) {
      int colorCodeInt = int.parse(secondaryColor);
      setState(() {
        secondaryColorTheme = Color(colorCodeInt);
        this.mounted;
      });
    } else {
      setState(() {
        secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
        this.mounted;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        body: isLoader
            ? Loader(
                loaderColor: Colors.red,
              )
            : Center(
                child: Column(
                  children: <Widget>[
                    DrawerMenuDesign(
                      onMenuClicked: widget.onMenuPressed,
                      isFilterEnabled: true,
                      isFilterClicked: () {
                        getStringValuesSF();
                        Navigator.push(
                            context,
                            MaterialPageRoute(
                                builder: (BuildContext context) =>
                                    ConditionalDropDown(
                                      theme: theme,
                                      secondaryColor: secondaryColorTheme,
                                      onCallBack: () {},
                                    )));


                      },
                    ),

                    //DrawerMenuDesign(),
                    Container(
                      height: MediaQuery.of(context).size.height * 0.79,
                      child: isLoader
                          ? Stack(
                              children: [
                                Loader(
                                  loaderColor: theme,
                                ),
                              ],
                            )
                          : Stack(
                              children: [
                                showGridView,
                                apiCallView,
                              ],
                            ),
                    ),
                  ],
                ),
              ),
      ),
    );
  }

  Widget get showGridView {
    /*return Container(
      child: InkWell(
        onTap: () {
          setState(() {
            // widget.isGridView = !widget.isGridView;

            changeOrderView();
          });
        },
        child:


        Container(
            alignment: Alignment.topRight,
            padding: EdgeInsets.only(right: 20, top: 10),
            child: Icon(
              !isGridView ? Icons.list : Icons.grid_view,
              semanticLabel:
                  isGridView ? "Change to List View" : "Change to Grid View",
              color: secondaryColorTheme ?? theme,
            )),
      ),
    );*/

    return Container(
      alignment: Alignment.topRight,
      padding: EdgeInsets.all(5),
      child: ClipRRect(
        borderRadius: BorderRadius.all(Radius.circular(32.0)),
        child: Material(
          shadowColor: Colors.transparent,
          color: Colors.transparent,
          child: IconButton(
            icon: Icon(
              !isGridView ? Icons.list : Icons.grid_view,
              semanticLabel:
                  isGridView ? "Change to List View" : "Change to Grid View",
              color: secondaryColorTheme,
            ),
            onPressed: () {
              changeOrderView();
            },
          ),
        ),
      ),
    );
  }

  Widget get apiCallView {
    return Container(
        margin: EdgeInsets.only(top: 45),
        padding: EdgeInsets.only(top: 10, bottom: 20),
        child: GridView.count(
          crossAxisCount: countValue,
          childAspectRatio: (aspectWidth / aspectHeight),
          children: List.generate(
              dashBoardDetailsBeanResponse.data.dashboard.length, (index) {
            return BUICard.dashBoardCardDeals(
              cardTitle: dashboard[index].name.toString(),
              currentMonthPrice:
                  "${MyHomePageState.buiFormatter.buiNumberFormatConversionWithCountryCode(dashboard[index].currentMonth, "In")}",
              lastMonthPrice:
                  "${MyHomePageState.buiFormatter.buiNumberFormatConversionWithCountryCode(dashboard[index].lastMonth, "in")}",
              trandValue:
                  dashboard[index].currentMonth >= dashboard[index].lastMonth
                      ? true
                      : false,
              cardDate: "(" +
                  "${MyHomePageState.buiFormatter.getPreviousMonthNameAndYear(now.month)}" +
                  ")",
              cardType: CardType.cardDashBoardDeal,
              cardSize: CardSize.small,
              width: MediaQuery.of(context).size.width,
              height: UniversalPlatform.isWeb
                  ? 30
                  : MediaQuery.of(context).size.height * 0.1,
              padding: 5,
              elevation: 30,
              shape: RoundedRectangleBorder(
                borderRadius: BorderRadius.circular(5),
              ),
            );
          }),
        ));
  }

  Future<void> changeOrderView() async {
    setState(() {
      isGridView = !isGridView;
    });

    changeMode();
  }

  changeMode() {
    if (countValue == 2) {
      setState(() {
        countValue = 1;
        aspectWidth = 3;
        aspectHeight = 1;
      });
    } else {
      setState(() {
        countValue = 2;
        aspectWidth = 2;
        aspectHeight = 1;
      });
    }
  }

  Future<void> onPopUpMenuFunction(int s) async {
    switch (s) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;
      case 6:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (BuildContext context) => ThemePickerScreen(
                  onBackPressed: onBackPressed,
                )));
  }

  void onBackPressed() {
    getStringValuesSF();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Toast.show("Message Pressed", context);
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {
      Navigator.push(context,
          MaterialPageRoute(builder: (BuildContext context) => CardStyle()));
    });
  }

  Future<void> onSettingFunction() async {
    setState(() {
      Toast.show("Setting Pressed", context);

      Navigator.push(context,
          MaterialPageRoute(builder: (BuildContext context) => ButtonView()));
    });
  }

  Future<void> onNotificationFunction() async {
    setState(() {
      Toast.show("Notification Pressed", context);

      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) => AppForceUpdate()));
    });
  }

  Future<void> onFilterClicked() async {
    setState(() {
      /*   jsonFormFilterData = json.encode({
        'fields': [
          {
            'apiKeyValue': 'startDate',
            'key': 'date',
            'type': 'date',
            'label': 'Start Date',
            'required': false,
          },
          {
            'apiKeyValue': 'endDate',
            'key': 'date',
            'type': 'date',
            'label': 'End Date'
          },


          {
            'apiKeyValue': 'industry',
            'key': 'Industry',
            'type': 'select',
            'label': 'Industry',
            'required': true,
            'items': industry != null
                ? industry
                : [
              {'label': '1', 'value': '2'}
            ]
          },

           {
            'apiKeyValue': 'statusid',
            'key': 'Status',
            'type': 'select',
            'label': 'Status',
            'required': true,
            'value': '1',
            'items': status != null
                ? status
                : [
              {'label': 'New', 'value': '1'},
              {'label': 'Active', 'value': '2'},
              {'label': 'Inactive', 'value': '3'},
            ]
          },
              {
            'apiKeyValue': 'industry',
            'key': 'select1',
            'type': 'select',
            'label': 'Industry',
            'required': false,
            'value': '',
            'items': [
              {
                'label': "Select",
                'value': "",
              },
              {"label": "Software", "value": "41"},
              {"label": "Hardware", "value": "42"},
              {"label": "Consumar goods", "value": "43"},
              {"label": "Farm & Farm Equipments", "value": "44"},
              {"label": "Manufacturing", "value": "45"},
              {"label": "Foundry", "value": "46"},
              {"label": "Automobile", "value": "47"},
              {"label": "Textile", "value": "48"},
              {"label": "Others", "value": "49"},
            ],
          },
          {
            'apiKeyValue': 'statusid',
            'key': 'select1',
            'type': 'select',
            'label': 'Status',
            'required': false,
            'items': [
              {"label": "New", "value": "6"},
              {"label": "Active", "value": "7"},
              {"label": "Inactive", "value": "8"},
              {'label': "Select", 'value': ""}
            ],
            "value": ""
          },
        ]
      });*/
      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) => FilterScreen(
                    primaryColor: theme,
                    secondaryColor: secondaryColorTheme,
                    filterFormData: jsonFormFilterData,
                    fromPageId: "accountId",
                    isActionApply: (String obj) {
                      callFilterChangesAPI(obj);
                    },
                  )));
    });
  }

  Future<void> callFilterChangesAPI(String obj) async {}

  Future<void> accountMastViewAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_MASTER_VIEW;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);

    if (detailsBean.statusCode == 401) {}
    if (detailsBean.status) {
      status = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Account")
            if (detailsBean.data[i].key == "Status")
              {
                "'label'": json.encode(detailsBean.data[i].name),
                "'value'": json.encode(detailsBean.data[i].id.toString()),
              },
      ];
      industry = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].key == "Industry")
            {
              json.encode('label'): json.encode(detailsBean.data[i].name),
              json.encode('value'):
                  json.encode(detailsBean.data[i].id.toString()),
            },
      ];

      status
          .add({"'label'": json.encode("Select"), "'value'": json.encode("")});

      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  final List<String> entries = <String>['A', 'B', 'C'];
  List<String> actions = ["Edit", "Delete"];
  final List<int> colorCodes = <int>[600, 500, 100];
  AccountDetailsBean accountDetailsBean;

  List<AccountListBean> accountListBeanData = new List();

  bool isLoader = true;

  String jsonFormFilterData;

  List<dynamic> status = new List();
  List<dynamic> industry = new List();

  Future<void> callDashBoardAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_DASHBOARD_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    DashBoardDetailBean dashBoardDetailBean = DashBoardDetailBean.fromJson(map);

    if (dashBoardDetailBean.status) {
      dashBoardDetailsBeanResponse = dashBoardDetailBean;
      dashboard = dashBoardDetailBean.data.dashboard;
      setState(() {
        isLoader = false;
      });
      Utils.showSnackBar(context, dashBoardDetailBean.message);
    }

    if (dashBoardDetailBean.statusCode == 401) {
      if (dashBoardDetailBean.message == AppStrings.UPDATE_APP) {
        setState(() {
          isLoader = false;
        });
      }
    }
  }
}
