import 'dart:convert';

import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/SvgPictures.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:bui_component/CustomAlertComponent/CustomeAlert.dart';
import 'package:bui_component/DataModalComponent/SlidableListView.dart';
import 'package:bui_component/ShimmerComponent/flutter_shimmer_loading_widget.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Models/leads_details_bean.dart';
import 'package:example/Utils/DrawerMenuDesign.dart';
import 'package:example/Utils/loader.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';

import '../ButtonView.dart';
import '../CardStyle.dart';
import '../ThemePickerScreen.dart';
import 'FilterScreen.dart';
import 'appUpdate.dart';

class LeadsPage extends DrawerContent {
  LeadsPage({Key key, this.title});

  final String title;

  @override
  _LeadsPageState createState() => _LeadsPageState();
}

class _LeadsPageState extends State<LeadsPage> {
  Color theme = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
  SharedPreferences prefs;

  final List<int> colorCodes = <int>[600, 500, 100];
  LeadsDetailsBean leadDetailsBean;

  List<LeadsListBean> leadListBeanData = new List();
  List<String> actions = ["Edit", "Delete"];

  bool isLoader = true;

  String jsonFormFilterData;

  List<dynamic> status = new List();
  List<dynamic> industry = new List();

  @override
  void initState() {
    // TODO: implement initState

    callPostAPI();
    getStringValuesSF();

    super.initState();
  }

  getStringValuesSF() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);
      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }
    String secondaryColor = prefs.getString(AppStrings.SECONDARY_THEME_COLOR);
    if (secondaryColor != null) {
      int colorCodeInt = int.parse(secondaryColor);
      setState(() {
        secondaryColorTheme = Color(colorCodeInt);
        this.mounted;
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
        secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
        this.mounted;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: isLoader
          ? Loader(
              loaderColor: Colors.red,
            )
          : Center(
              child: Column(
                children: <Widget>[
                  /*   Container(
                    color: theme ?? AppColors.primaryColor,
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceBetween,
                      children: <Widget>[
                        ClipRRect(
                          borderRadius: BorderRadius.all(Radius.circular(32.0)),
                          child: Material(
                            shadowColor: Colors.transparent,
                            color: Colors.transparent,
                            child: IconButton(
                              icon: Icon(
                                Icons.menu,
                                color: Colors.white,
                              ),
                              onPressed: () {
                                widget.onMenuPressed();
                              },
                            ),
                          ),
                        ),
                        Row(
                          children: [
                            Visibility(
                              visible: true,
                              child: InkWell(
                                onTap: () {
                                  onFilterClicked();
                                },
                                child: Container(
                                  //padding: EdgeInsets.only(right: 2),
                                  child: Icon(
                                    Icons.filter_alt,
                                    color: AppColors.appWhite,
                                  ),
                                ),
                              ),
                            ),
                            Container(
                                //padding: EdgeInsets.only(right: 2),
                                child: PopupMenuButton(
                              elevation: 30,
                              //offset: Offset(0, 100),
                              tooltip: "More Feature",
                              itemBuilder: (context) => [
                                PopupMenuItem(
                                  height: 30,
                                  value: 1,
                                  child: Row(
                                    children: [
                                      Padding(
                                        padding: const EdgeInsets.all(12.0),
                                        child: Icon(
                                          Icons.settings,
                                          color: theme ??
                                              AppColors.APP_COLOR_BLACK,
                                        ),
                                      ),
                                      Text(
                                        AppStrings.SETTINGS,
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            fontSize: FontSize.FONT_14,
                                            color: theme ?? Colors.black87),
                                      )
                                    ],
                                  ),
                                ),
                                PopupMenuItem(
                                  value: 2,
                                  child: Row(
                                    children: [
                                      Padding(
                                        padding: const EdgeInsets.all(12.0),
                                        child: Icon(
                                          Icons.notifications_active,
                                          color: theme ??
                                              AppColors.APP_COLOR_BLACK,
                                        ),
                                      ),
                                      Text(
                                        AppStrings.NOTIFICATION,
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            fontSize: FontSize.FONT_14,
                                            color: theme ?? Colors.black87),
                                      )
                                    ],
                                  ),
                                ),
                                PopupMenuItem(
                                  value: 3,
                                  child: Row(
                                    children: [
                                      Padding(
                                        padding: const EdgeInsets.all(12.0),
                                        child: Icon(
                                          Icons.messenger_rounded,
                                          color: theme ??
                                              AppColors.APP_COLOR_BLACK,
                                        ),
                                      ),
                                      Text(
                                        AppStrings.MESSAGE,
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            fontSize: FontSize.FONT_14,
                                            color: theme ?? Colors.black87),
                                      )
                                    ],
                                  ),
                                ),
                                PopupMenuItem(
                                  value: 4,
                                  child: Row(
                                    children: [
                                      Padding(
                                        padding: const EdgeInsets.all(12.0),
                                        child: Icon(
                                          Icons.color_lens_sharp,
                                          color: theme ??
                                              AppColors.APP_COLOR_BLACK,
                                        ),
                                      ),
                                      Text(
                                        AppStrings.THEME,
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            fontSize: FontSize.FONT_14,
                                            color: theme ?? Colors.black87),
                                      )
                                    ],
                                  ),
                                ),
                                PopupMenuItem(
                                  value: 5,
                                  child: Row(
                                    children: [
                                      Padding(
                                        padding: const EdgeInsets.all(12.0),
                                        child: Icon(
                                          Icons.feedback_rounded,
                                          color: theme ??
                                              AppColors.APP_COLOR_BLACK,
                                        ),
                                      ),
                                      Text(
                                        AppStrings.FEED_BACK,
                                        style: TextStyle(
                                            fontWeight: FontWeight.bold,
                                            fontSize: FontSize.FONT_14,
                                            color: theme ?? Colors.black87),
                                      )
                                    ],
                                  ),
                                ),
                              ],
                              onCanceled: () {},
                              onSelected: (value) {
                                onPopUpMenuFunction(value);
                              },
                              icon: Icon(
                                Icons.more_vert_sharp,
                                color: AppColors.appWhite,
                              ),
                            )),
                          ],
                        ),
                      ],
                    ),
                  ),*/
                  DrawerMenuDesign(
                    onMenuClicked: widget.onMenuPressed,
                    isFilterEnabled: true,
                    isFilterClicked: onFilterClicked,
                  ),
                  isLoader
                      ? Stack(
                          children: [
                            deleteDetailsView,
                            isLoaderView,
                          ],
                        )
                      : Stack(
                          children: [deleteDetailsView, detailsView],
                        )
                ],
              ),
            ),
    );
  }

  Widget get isLoaderView {
    return Container(
        child: ListView.builder(
      itemCount: 100,
      itemBuilder: (context, index) {
        return Column(
          children: [
            SizedBox(
              height: 20,
            ),
            Card(
              elevation: 10,
              shape: BeveledRectangleBorder(
                  borderRadius: BorderRadius.only(
                bottomLeft: Radius.circular(15.0),
                bottomRight: Radius.circular(15.0),
                topLeft: Radius.circular(15.0),
                topRight: Radius.circular(15.0),
              )),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  SizedBox(
                    height: 20,
                  ),
                  Container(
                    height: 50,
                    child: Padding(
                      padding: const EdgeInsets.only(left: 10, right: 10),
                      child: FlutterShimmnerLoadingWidget(
                        count: 2,
                        animate: true,
                        color: Colors.grey[200],
                      ),
                    ),
                  ),
                  Container(
                    decoration: BoxDecoration(
                        color: Colors.grey[200],
                        shape: BoxShape.rectangle,
                        borderRadius: BorderRadius.only(
                            bottomLeft: Radius.circular(15.0),
                            bottomRight: Radius.circular(15.0))),
                    alignment: Alignment.bottomRight,
                    padding: EdgeInsets.all(10),
                    child: FlutterShimmnerLoadingWidget(
                      count: 1,
                      animate: true,
                      color: Colors.white,
                      align: TextAlign.right,
                      maxWidth: 0.30,
                      minWidth: 0.30,
                      //lineHeight: 10,
                    ),
                  )
                ],
              ),
            ),
          ],
        );
      },
    ));
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      margin: EdgeInsets.only(top: 0),
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
        "Swipe left to use the edit and delete option",
        style: TextStyle(
            // fontWeight: FontWeight.bold,
            fontSize: FontSize.FONT_14,
            color: Colors.black87),
      ),
      color: Colors.grey[400],
    );
  }

  Widget get detailsView {
    if (leadListBeanData.length != 0) {
      return Container(
        height: MediaQuery.of(context).size.height * 0.85,
        child: SlideListView(
          itemBuilder: (bc, index) {
            return InkWell(
              onTap: () {
                Toast.show(leadListBeanData[index].email, context);
              },
              child: BUICard.dashBoardCard(
                width: MediaQuery.of(context).size.width,
                cardSize: CardSize.large,
                cardTitle: leadListBeanData[index].firstname ?? "",
                cardName: leadListBeanData[index].companyname ?? "",
                statusTag: leadListBeanData[index].statusid.name ?? "",
                dashBoardCardTime: leadListBeanData[index].createdAt != null
                    ? leadListBeanData[index].createdAt.substring(0, 10)
                    : "",
                height: MediaQuery.of(context).size.height * 0.26,
                elevation: 5,
                tagValue: leadListBeanData[index].tags.split(',').toList(),
                isTagValue: true,
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(8.0),
                ),
                //tagValue: ["#one"],
              ),
            );
          },
          actionWidgetDelegate:
              ActionWidgetDelegate(actions.length, (actionIndex, listIndex) {
            if (actionIndex == 0) {
              return Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Icon(
                    Icons.edit,
                    color: theme ?? AppColors.primaryColor,
                  ),
                  Text(
                    actions[0].toString(),
                    overflow: TextOverflow.ellipsis,
                    style: TextStyle(
                      fontWeight: FontWeight.bold,
                      fontSize: 14,
                      color: theme ?? AppColors.primaryColor,
                    ),
                  ),
                ],
              );
            } else {
              return Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Icon(Icons.delete, color: AppColors.errorWarning),
                  Text(
                    actions[1].toString(),
                    overflow: TextOverflow.ellipsis,
                    style: TextStyle(
                        fontWeight: FontWeight.bold,
                        fontSize: 12,
                        color: AppColors.errorWarning),
                  ),
                ],
              );
            }
          }, (int indexInList, int index, BaseSlideItem item) {
            if (index == 1) {
              //item.remove();

              CustomAlertBox.showCustomAlertBox(
                context: context,
                type: "Warning",
                enterWidget:
                    Text("Are you sure do you want to delete this Item"),
                isPositiveEnabled: true,
                isNegativeEnabled: true,
                setPositiveAction: () {
                  item.remove();
                  // Navigator.pop(context);
                },
              );
              //  _showMyDialog(indexInList, item);
            } else {
              item.close();
              CustomAlertBox.showCustomAlertBox(
                context: context,
                type: "Alert",
                enterWidget: Text("This is Sample Alert"),
                isPositiveEnabled: true,
                isNegativeEnabled: false,
                positiveButtonColor: Colors.green,
                setPositiveAction: () {},
              );
              Toast.show("Close this Item", context);
            }
          }, [Colors.white, Colors.white]),
          dataList: leadListBeanData,
          itemBackgroundColor: Colors.grey[200],
          refreshCallback: () async {
            await Future.delayed(Duration(seconds: 2));

            setState(() {
              // fetchData();

              callPostAPI();

              this.mounted;
            });
            return;
          },
        ),
      );
    } else {
      return Container(
          margin: EdgeInsets.only(top: 90),
          child: Center(
            child: Column(
              mainAxisAlignment: MainAxisAlignment.center,
              children: [
                SvgPicture.asset(
                  SvgPictures.NO_DATA_FOUND,
                  placeholderBuilder: (context) => CircularProgressIndicator(),
                  height: 128.0,
                  allowDrawingOutsideViewBox: false,
                ),
                SizedBox(
                  height: 20,
                ),
                Text(
                  "OOPS, There is no data found",
                  style: TextStyle(
                      fontWeight: FontWeight.bold,
                      fontSize: FontSize.FONT_20,
                      color: AppColors.APP_GRAY_1),
                ),
              ],
            ),
          ));
    }
  }

  Future<void> onPopUpMenuFunction(int s) async {
    switch (s) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;
      case 6:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (BuildContext context) => ThemePickerScreen(
                  onBackPressed: onBackPressed,
                )));
  }

  void onBackPressed() {
    getStringValuesSF();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Toast.show("Message Pressed", context);
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {
      Navigator.push(context,
          MaterialPageRoute(builder: (BuildContext context) => CardStyle()));
    });
  }

  Future<void> onSettingFunction() async {
    setState(() {
      Toast.show("Setting Pressed", context);

      Navigator.push(context,
          MaterialPageRoute(builder: (BuildContext context) => ButtonView()));
    });
  }

  Future<void> onFilterClicked() async {
    setState(() {
      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) => FilterScreen(
                    primaryColor: theme,
                    secondaryColor: secondaryColorTheme,
                    filterFormData: jsonFormFilterData,
                    fromPageId: "leadsId",
                    isActionApply: (String obj) {
                      callFilterChangesAPI(obj);
                    },
                  )));
    });
  }

  Future<void> onNotificationFunction() async {
    setState(() {
      Toast.show("Notification Pressed", context);

      Navigator.push(
          context,
          MaterialPageRoute(
              builder: (BuildContext context) => AppForceUpdate()));
    });
  }

  Future<void> callFilterChangesAPI(String obj) async {
    callPostAPI(obj: obj);
  }

  Future<void> accountMastViewAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_MASTER_VIEW;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);

    if (detailsBean.statusCode == 401) {}
    if (detailsBean.status) {
      status = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Account")
            if (detailsBean.data[i].key == "Status")
              {
                "'label'": json.encode(detailsBean.data[i].name),
                "'value'": json.encode(detailsBean.data[i].id.toString()),
              },
      ];
      industry = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].key == "Industry")
            {
              json.encode('label'): json.encode(detailsBean.data[i].name),
              json.encode('value'):
                  json.encode(detailsBean.data[i].id.toString()),
            },
      ];

      status
          .add({"'label'": json.encode("Select"), "'value'": json.encode("")});

      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> callPostAPI({String obj}) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_LEADS_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    LeadsDetailsBean leadDetailsResponseBean = LeadsDetailsBean.fromJson(map);

    if (leadDetailsResponseBean.statusCode == 400) {
      setState(() {
        isLoader = true;
      });
    }

    if (leadDetailsResponseBean.status) {
      leadDetailsBean = leadDetailsResponseBean;
      leadListBeanData = leadDetailsResponseBean.data;

      status = [
        for (int i = 0; i < leadListBeanData.length; i++)
          if (leadListBeanData[i].statusid != null)
            {
              'label': leadListBeanData[i].statusid.name,
              'value': leadListBeanData[i].statusid.id.toString(),
            },
      ];
      industry = [
        for (int i = 0; i < leadListBeanData.length; i++)
          if (leadListBeanData[i].industry != null)
            {
              'label': leadListBeanData[i].industry.name,
              'value': leadListBeanData[i].industry.id.toString(),
            },
      ];

      /* sales = [
        for (int i = 0; i < leadListBeanData.length; i++)
          if (leadListBeanData[i].salesarea != null)
            {
              'label': leadListBeanData[i].salesarea.name.toString(),
              'value': leadListBeanData[i].salesarea.id.toString(),
            },
      ];*/

      status.insert(0, {
        'label': '',
        'value': '',
      });
      industry.insert(0, {
        'label': '',
        'value': '',
      });

      final statusList = status.map((item) => jsonEncode(item)).toList();

      final uniqueStatusJsonList = statusList.toSet().toList();

      status = uniqueStatusJsonList.map((item) => jsonDecode(item)).toList();

      final industryList = industry.map((item) => jsonEncode(item)).toList();

      final uniqueIndustryList = industryList.toSet().toList();

      industry = uniqueIndustryList.map((item) => jsonDecode(item)).toList();

      jsonFormFilterData = json.encode({
        'fields': [
          {
            'apiKeyValue': 'industry',
            'key': 'select1',
            'type': 'select',
            'label': 'Industry',
            'required': false,
            'value': '',
            'items': industry ??
                [
                  {
                    'label': "Select",
                    'value': "",
                  },
                  {
                    'value': "1",
                    'label': "Hardware",
                  },
                  {
                    'value': "2",
                    'label': "Software",
                  },
                  {
                    'value': "3",
                    'label': "Comsumer goal",
                  },
                  {
                    'value': "4",
                    'label': "Others",
                  },
                ]
          },
          {
            'apiKeyValue': 'statusid',
            'key': 'select1',
            'type': 'select',
            'label': 'Status',
            'required': false,
            'value': '',
            'items': status ??
                [
                  {
                    'label': "Select",
                    'value': "",
                  },
                  {
                    'value': "1",
                    'label': "New",
                  },
                  {
                    'value': "2",
                    'label': "Active",
                  },
                  {
                    'value': "3",
                    'label': "Inactive",
                  },
                  {
                    'value': "4",
                    'label': "Hot",
                  },
                ]
          },
        ]
      });

      setState(() {
        isLoader = false;
      });
    }
  }
}
