import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CustomAlertComponent/CustomeAlert.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/ImageAssets.dart';
import 'package:example/Firebase%20Classes/sign_in.dart';
import 'package:example/Screen/ERP/ContactPage.dart';
import 'package:example/Screen/ERP/Home.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'DashBoardScreen.dart';
import 'LeadsPage.dart';
import 'TaskScreen.dart';

class MainWidget extends StatefulWidget {
  MainWidget({Key key, this.title}) : super(key: key);
  final String title;

  @override
  _MainWidgetState createState() => _MainWidgetState();
}

class _MainWidgetState extends State<MainWidget> with TickerProviderStateMixin {
  HiddenDrawerController _drawerController;
  Color theme = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
  SharedPreferences prefs;

  int selectedIndex;

  @override
  void initState() {
    super.initState();

    colorCodePrefs();
    _drawerController = HiddenDrawerController(
      initialPage: DashboardPage(
        title: 'main',
      ),
      items: [
        DrawerItem(
          text: Text('Dashboard', style: TextStyle(color: Colors.white)),
          icon: Icon(Icons.dashboard, color: Colors.white),
          page: DashboardPage(
            title: 'Dashboard',
          ),
        ),
        DrawerItem(
          text: Text('Leads', style: TextStyle(color: Colors.white)),
          icon: Icon(Icons.leaderboard, color: Colors.white),
          page: LeadsPage(
            title: 'Leads',
          ),
        ),
        DrawerItem(
          text: Text(
            'Accounts',
            style: TextStyle(color: Colors.white),
          ),
          icon: Icon(Icons.account_box, color: Colors.white),
          page: MainPage(
            title: 'Notification',
          ),
        ),
        DrawerItem(
          text: Text(
            'Contacts',
            style: TextStyle(color: Colors.white),
          ),
          icon: Icon(Icons.contact_page, color: Colors.white),
          page: ContactPage(
            title: 'Contacts',
          ),
        ),
        DrawerItem(
          text: Text(
            'Task',
            style: TextStyle(color: Colors.white),
          ),
          icon: Icon(Icons.task, color: Colors.white),
          page: TaskScreen(
            title: 'Task',
          ),
        ),
      ],
    );
  }

  Future<void> colorCodePrefs() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);
      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }
    String secondaryColor = prefs.getString(AppStrings.SECONDARY_THEME_COLOR);
    if (secondaryColor != null) {
      int colorCodeInt = int.parse(secondaryColor);
      setState(() {
        secondaryColorTheme = Color(colorCodeInt);
        this.mounted;
      });
    } else {
      setState(() {
        secondaryColorTheme = AppColors.APP_COLOR_PRIMARY;
        this.mounted;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      floatingActionButtonLocation: FloatingActionButtonLocation.centerDocked,
      floatingActionButton: FloatingActionButton(
        mini: true,
        backgroundColor: Colors.white,
        splashColor: Colors.green,
        child: Icon(
          selectedIndex != 1 ? Icons.add : Icons.grid_view,
          color: Colors.black,
        ),
        elevation: 50,
        onPressed: () {
          setState(() {
            print("Selected Index====" + selectedIndex.toString());
          });
        },
      ),
      bottomNavigationBar: new BottomAppBar(
        shape: CircularNotchedRectangle(),
        notchMargin: 5,
        color: AppColors.primaryColor,
        child: new Row(
          mainAxisSize: MainAxisSize.max,
          children: <Widget>[
            new Expanded(child: new SizedBox()),
            IconButton(
              icon: Icon(
                Icons.dashboard,
                color: Colors.white,
              ),
              onPressed: () {
                setState(() {
                  selectedIndex = 1;
                  print("Clicked");
                });
              },
            ),
            new Expanded(child: new SizedBox()),
            IconButton(
              icon: Icon(
                Icons.leaderboard,
                color: Colors.white,
              ),
              onPressed: () {
                setState(() {
                  selectedIndex = 2;
                  DrawerItem(
                    text: Text(
                      'Task',
                      style: TextStyle(color: Colors.white),
                    ),
                    icon: Icon(Icons.task, color: Colors.white),
                    page: TaskScreen(
                      title: 'Task',
                    ),
                  );
                });
              },
            ),
            new Expanded(child: new SizedBox()),
            IconButton(
              icon: Icon(
                Icons.account_circle_sharp,
                color: Colors.white,
              ),
              onPressed: () {
                setState(() {
                  selectedIndex = 3;
                });
              },
            ),
            new Expanded(child: new SizedBox()),
            IconButton(
              icon: Icon(
                Icons.contacts,
                color: Colors.white,
              ),
              onPressed: () {
                setState(() {
                  selectedIndex = 4;
                });
              },
            ),
            new Expanded(child: new SizedBox()),
          ],
        ),
      ),
      body: HiddenDrawer(
        controller: _drawerController,
        header: Column(
          mainAxisAlignment: MainAxisAlignment.spaceBetween,
          children: [
            Container(
              alignment: Alignment.topLeft,
              padding: EdgeInsets.only(left: 30),
              child:
                  /*CircleAvatar(
                radius: 50,
                backgroundImage: NetworkImage(
                    'https://helpx.adobe.com/content/dam/help/en/photoshop/using/convert-color-image-black-white/jcr_content/main-pars/before_and_after/image-before/Landscape-Color.jpg'),
              ),*/

                  CircleAvatar(
                radius: 70,
                backgroundColor: Colors.white,
                child: Container(
                  padding: EdgeInsets.all(20),
                  child: Image.asset(
                    ImageAssets.LOGO_WITH_NAME,
                    height: 150,
                  ),
                ),
              ),
            ),
          ],
        ),
        decoration: BoxDecoration(
          gradient: LinearGradient(
            begin: Alignment.topRight,
            end: Alignment.bottomLeft,
            colors: [
              Colors.black,
              Colors.black,
              Colors.black87,
              Colors.black87,
            ],
            // tileMode: TileMode.repeated,
          ),
        ),
        logout: () {
          signOutFunction(context);
        },
        versionName: "0.7.8",
      ),
    );
  }

  Future<void> signOutFunction(BuildContext context) async {
    CustomAlertBox.showCustomAlertBox(
        context: context,
        type: AppStrings.ARE_U_SURE,
        enterWidget: Text(AppStrings.DO_U_WANT_LOGOUT),
        isPositiveEnabled: true,
        isNegativeEnabled: true,
        positiveButtonColor: AppColors.primaryColor,
        setPositiveAction: () async {
          await FirebaseAuth.instance.signOut();
          await signOutGoogle(context);
        },
        positiveButtonTextColor: Colors.white);
  }
}
