import 'dart:convert';

import 'package:bui_component/BuiDrawerComponent/buiDrawer.dart';
import 'package:bui_component/CardComponent/cardComponent.dart';
import 'package:bui_component/CustomAlertComponent/CustomeAlert.dart';
import 'package:bui_component/DataModalComponent/SlidableListView.dart';
import 'package:bui_component/ShimmerComponent/flutter_shimmer_widget.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/task_detail_bean.dart';
import 'package:example/Screen/ERP/TaskViewDetailScreenbk.dart';
import 'package:example/Utils/DrawerMenuDesign.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import '../ThemePickerScreen.dart';
import 'ConditionalDropDown.dart';
import 'FilterScreen.dart';

class TaskScreen extends DrawerContent {
  TaskScreen({Key key, this.title});

  final String title;

  @override
  _TaskScreenState createState() => _TaskScreenState();
}

class _TaskScreenState extends State<TaskScreen> {
  Color theme = AppColors.APP_COLOR_PRIMARY;
  Color secondaryColor;

  SharedPreferences prefs;

  TaskDetailBean taskDetailBean;

  List<TaskDetailList> taskListBeanData = new List();

  bool isLoader = true;

  String jsonFormFilterData;

  List<String> actions = ["Edit", "Delete"];

  List<dynamic> account = new List();
  List<dynamic> status = new List();
  List<dynamic> contact = new List();
  List<dynamic> lead = new List();
  List<dynamic> activityType = new List();
  List<dynamic> deal = new List();

  @override
  void initState() {
    taskScreenViewDetailsAPI();
    colorCodePref();
    super.initState();
  }

  var now = DateTime.now();

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        body: Center(
          child: Column(
            children: [
              DrawerMenuDesign(
                onMenuClicked: widget.onMenuPressed,
                isFilterEnabled: true,
                isFilterClicked: onFilterClicked,
              ),
              isLoader
                  ? Stack(
                      children: [
                        deleteDetailsView,
                        isLoaderView,
                      ],
                    )
                  : Stack(
                      children: [deleteDetailsView, detailsView],
                    )
            ],
          ),
        ),
        floatingActionButton: FloatingActionButton(
          onPressed: () {
            /* Navigator.of(context).push(MaterialPageRoute(builder: (context) {
              return TaskViewDetailScreen(
                pageTitle: "Info",
                type: "Add",
                theme: theme,
                onCallBack: onCallBackAPI,
                secondaryColor: secondaryColor,
              );
            }));*/

            Navigator.push(
                context,
                MaterialPageRoute(
                    builder: (BuildContext context) => ConditionalDropDown(
                          theme: theme,
                          secondaryColor: secondaryColor,
                          onCallBack: onCallBackAPI,
                        )));
          },
          child: const Icon(Icons.add),
          backgroundColor: theme,
        ),
      ),
    );
  }

  Widget get isLoaderView {
    return Container(
        height: MediaQuery.of(context).size.height * 0.80,
        margin: EdgeInsets.only(top: 30),
        child: ListView.builder(
          itemCount: 100,
          itemBuilder: (context, index) {
            return Column(
              children: [
                SizedBox(
                  height: 20,
                ),
                Card(
                  elevation: 10,
                  shape: BeveledRectangleBorder(
                      borderRadius: BorderRadius.only(
                          bottomLeft: Radius.circular(15.0),
                          bottomRight: Radius.circular(15.0))),
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      SizedBox(
                        height: 20,
                      ),
                      Container(
                        height: 50,
                        child: Padding(
                          padding: const EdgeInsets.only(left: 10, right: 10),
                          child: FlutterShimmnerLoadingWidget(
                            count: 2,
                            animate: true,
                            color: Colors.grey[200],
                          ),
                        ),
                      ),
                      Container(
                        decoration: BoxDecoration(
                            color: Colors.grey[200],
                            shape: BoxShape.rectangle,
                            borderRadius: BorderRadius.only(
                                bottomLeft: Radius.circular(15.0),
                                bottomRight: Radius.circular(15.0))),
                        alignment: Alignment.bottomRight,
                        padding: EdgeInsets.all(10),
                        child: FlutterShimmnerLoadingWidget(
                          count: 1,
                          animate: true,
                          color: Colors.white,
                          align: TextAlign.right,
                          maxWidth: 0.30,
                          minWidth: 0.30,
                        ),
                      )
                    ],
                  ),
                ),
              ],
            );
          },
        ));
  }

  Widget get deleteDetailsView {
    return Container(
      height: 30,
      //
      width: MediaQuery.of(context).size.width,
      alignment: Alignment.center,
      child: Text(
        "Swipe left to use the edit and delete option",
        style: TextStyle(fontSize: 14, color: Colors.black87),
      ),
      color: Colors.grey[400],
    );
  }

  Widget get addDetailsView {
    return Container(
      height: 60,
      padding: EdgeInsets.all(5),
      alignment: Alignment.center,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.end,
        children: [
          Container(
            height: 50,
            width: 50,
            padding: EdgeInsets.all(5),
            child: ClipOval(
              child: Material(
                color: theme, // button color
                child: InkWell(
                  splashColor: Colors.white, // inkwell color
                  child: SizedBox(
                      width: 56,
                      height: 56,
                      child: Icon(Icons.add, color: Colors.white)),
                  onTap: () {
                    taskScreenViewDetailsAPI();
                  },
                ),
              ),
            ),
          ),
        ],
      ),
    );
  }

  Widget get detailsView {
    if (taskListBeanData.length != 0) {
      return Container(
        height: MediaQuery.of(context).size.height * 0.80,
        margin: EdgeInsets.only(top: 30),
        child: SlideListView(
          itemBuilder: (bc, index) {
            return InkWell(
              onTap: () {
                Navigator.of(context).push(MaterialPageRoute(
                  builder: (context) {
                    return TaskViewDetailScreen(
                      pageTitle: "Info",
                      id: taskListBeanData[index].id,
                      taskListBeanData: taskListBeanData[index],
                      theme: theme,
                      secondaryColor: secondaryColor,
                      onCallBack: onCallBackAPI,
                    );
                  },
                ));
              },
              child: BUICard.dashBoardCard(
                width: MediaQuery.of(context).size.width,
                cardSize: CardSize.large,
                cardTitle: taskListBeanData[index].activitytypeid.name != null
                    ? taskListBeanData[index].activitytypeid.name
                    : "",
                cardName: taskListBeanData[index].duedate != null
                    ? taskListBeanData[index].duedate
                    : "",
                statusTag: taskListBeanData[index].statusid.name != null
                    ? taskListBeanData[index].statusid.name
                    : "",
                dashBoardCardTime:
                    "${MyHomePageState.buiFormatter.buiDateTimeDDMMYYFormat(taskListBeanData[index].createdAt)}" ??
                        "",
                isTagValue: false,
                height: 130,
                elevation: 10,
                shape: RoundedRectangleBorder(
                  borderRadius: BorderRadius.circular(5.0),
                ),
              ),
            );
          },
          actionWidgetDelegate:
              ActionWidgetDelegate(actions.length, (actionIndex, listIndex) {
            if (actionIndex == 0) {
              return Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Icon(
                    Icons.edit,
                    color: theme ?? AppColors.primaryColor,
                  ),
                  Text(
                    actions[0].toString(),
                    overflow: TextOverflow.ellipsis,
                    style: TextStyle(
                      fontWeight: FontWeight.bold,
                      fontSize: 14,
                      color: theme ?? AppColors.primaryColor,
                    ),
                  ),
                ],
              );
            } else {
              return Column(
                mainAxisSize: MainAxisSize.min,
                children: <Widget>[
                  Icon(Icons.delete, color: AppColors.errorWarning),
                  Text(
                    actions[1].toString(),
                    overflow: TextOverflow.ellipsis,
                    style: TextStyle(
                        fontWeight: FontWeight.bold,
                        fontSize: 12,
                        color: AppColors.errorWarning),
                  ),
                ],
              );
            }
          }, (int indexInList, int index, BaseSlideItem item) {
            if (index == 1) {
              CustomAlertBox.showCustomAlertBox(
                context: context,
                type: "Warning",
                enterWidget:
                    Text("Are you sure do you want to delete this Item"),
                isPositiveEnabled: true,
                isNegativeEnabled: true,
                setPositiveAction: () {
                  setState(() {
                    item.remove();
                  });
                  callTasksDeleteAPI(
                      taskListBeanData[indexInList].id.toString(), item);
                },
              );
            } else {
              item.close();
              Navigator.of(context).push(MaterialPageRoute(
                builder: (context) {
                  return TaskViewDetailScreen(
                    pageTitle: "Info",
                    id: taskListBeanData[index].id,
                    taskListBeanData: taskListBeanData[index],
                    theme: theme,
                    secondaryColor: secondaryColor,
                    onCallBack: onCallBackAPI,
                  );
                },
              ));
            }
          }, [Colors.white, Colors.white]),
          dataList: taskListBeanData,
          itemBackgroundColor: Colors.grey[200],
          refreshCallback: () async {
            await Future.delayed(Duration(seconds: 2));

            setState(() {
              taskScreenViewDetailsAPI();

              this.mounted;
            });
            return;
          },
        ),
      );
    } else {
      return Container();
    }
  }

  Future<void> callContactsDeleteAPI(String index, BaseSlideItem item) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_DELETE_CONTACTS_URL;
    var objToSend = {
      "\"condition\"": [
        {
          "\"id\"": index,
        }
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    BaseResponse baseResponse = BaseResponse.fromJson(map);

    if (baseResponse.status) {
      taskScreenViewDetailsAPI();
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  colorCodePref() async {
    prefs = await SharedPreferences.getInstance();
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    String secondaryColorPref =
        prefs.getString(AppStrings.SECONDARY_THEME_COLOR);

    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }

    if (secondaryColorPref != null) {
      int colorCodeInt = int.parse(secondaryColorPref);
      setState(() {
        secondaryColor = Color(colorCodeInt);
        this.mounted;
      });

      return stringValue;
    } else {
      secondaryColor = AppColors.APP_COLOR_PRIMARY;
    }
  }

  Future<void> onAPICallBack() async {
    isLoader = false;

    setState(() {
      taskScreenViewDetailsAPI();
      this.mounted;
    });
  }

  void onBackPressed() {
    colorCodePref();
  }

  Future<void> onMessageFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Message Pressed");
    });
  }

  Future<void> onFeedBackFunction() async {
    setState(() {});
  }

  Future<void> onSettingFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Setting Pressed");
    });
  }

  Future<void> onNotificationFunction() async {
    setState(() {
      Utils.showSnackBar(context, "Notification Pressed");
    });
  }

  Future<void> onPopUpMenuPressed(int count) async {
    switch (count) {
      case 1:
        onSettingFunction();
        break;
      case 2:
        onNotificationFunction();
        break;
      case 3:
        onMessageFunction();
        break;
      case 4:
        onThemeFunction();
        break;
      case 5:
        onFeedBackFunction();
        break;

      default:
        break;
    }
  }

  Future<void> onThemeFunction() async {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (BuildContext context) => ThemePickerScreen(
                  onBackPressed: onBackPressed,
                )));
  }

  Future<void> onFilterClicked() async {
    colorCodePref();

    colorCodePref();
    setState(() {
      Navigator.of(context).push(MaterialPageRoute(
        builder: (context) {
          return FilterScreen(
            primaryColor: theme,
            secondaryColor: secondaryColor,
            filterFormData: jsonFormFilterData,
            fromPageId: "activityid",
            isActionApply: (String obj) {
              callFilterAPI(obj);
            },
          );
        },
      ));
      /*   Navigator.push(
          context,
          Transition(
              child: FilterScreen(
                primaryColor: theme,
                secondaryColor: secondaryColor,
                filterFormData: jsonFormFilterData,
                fromPageId: "activityid",
                isActionApply: (String obj) {
                  callFilterAPI(obj);
                },
              ),
              transitionEffect: TransitionEffect.RIGHT_TO_LEFT));*/
    });
  }

  Future<void> callTasksDeleteAPI(String index, BaseSlideItem item) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_DELETE_TASK_URL;
    var objToSend = {
      "\"condition\"": [
        {
          "\"id\"": index,
        }
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    BaseResponse baseResponse = BaseResponse.fromJson(map);

    if (baseResponse.status) {
      taskScreenViewDetailsAPI();
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> onCallBackAPI() async {
    isLoader = false;

    setState(() {
      taskScreenViewDetailsAPI();
      this.mounted;
    });
  }

  Future<void> taskScreenViewDetailsAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_TASK_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    TaskDetailBean contactDetailsResponseBean = TaskDetailBean.fromJson(map);

    if (contactDetailsResponseBean.status) {
      taskDetailBean = contactDetailsResponseBean;
      taskListBeanData = contactDetailsResponseBean.data;

      activityType = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].activitytypeid != null)
            {
              'label': taskListBeanData[i].activitytypeid.name,
              'value': taskListBeanData[i].activitytypeid.id.toString(),
            },
      ];
      status = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].statusid != null)
            {
              'label': taskListBeanData[i].statusid.name,
              'value': taskListBeanData[i].statusid.id.toString(),
            },
      ];
      account = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].accountid != null)
            {
              'label': taskListBeanData[i].accountid.companyname,
              'value': taskListBeanData[i].accountid.id.toString(),
            },
      ];
      contact = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].contactid != null)
            {
              'label': taskListBeanData[i].contactid.firstname,
              'value': taskListBeanData[i].contactid.id.toString(),
            },
      ];
      lead = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].leadid != null)
            {
              'label': taskListBeanData[i].leadid.firstname,
              'value': taskListBeanData[i].leadid.id.toString(),
            },
      ];
      deal = [
        for (int i = 0; i < taskListBeanData.length; i++)
          if (taskListBeanData[i].dealid != null)
            {
              'label': taskListBeanData[i].dealid.name,
              'value': taskListBeanData[i].dealid.id.toString(),
            },
      ];

      account.insert(0, {
        'label': '',
        'value': '',
      });
      contact.insert(0, {
        'label': '',
        'value': '',
      });
      deal.insert(0, {
        'label': '',
        'value': '',
      });
      lead.insert(0, {
        'label': '',
        'value': '',
      });
      activityType.insert(0, {
        'label': '',
        'value': '',
      });
      status.insert(0, {
        'label': '',
        'value': '',
      });

      final statusList = status.map((item) => jsonEncode(item)).toList();

      final uniqueStatusJsonList = statusList.toSet().toList();

      status = uniqueStatusJsonList.map((item) => jsonDecode(item)).toList();

      final activityTypeList =
          activityType.map((item) => jsonEncode(item)).toList();

      final uniqueActivityTypeList = activityTypeList.toSet().toList();

      activityType =
          uniqueActivityTypeList.map((item) => jsonDecode(item)).toList();

      final accountList = account.map((item) => jsonEncode(item)).toList();

      final uniqueAccountList = accountList.toSet().toList();

      account = uniqueAccountList.map((item) => jsonDecode(item)).toList();

      final contactList = contact.map((item) => jsonEncode(item)).toList();

      final uniqueContactList = contactList.toSet().toList();

      contact = uniqueContactList.map((item) => jsonDecode(item)).toList();

      final dealList = deal.map((item) => jsonEncode(item)).toList();

      final uniqueDealList = dealList.toSet().toList();

      deal = uniqueDealList.map((item) => jsonDecode(item)).toList();

      final leadList = lead.map((item) => jsonEncode(item)).toList();

      final uniqueLeadList = leadList.toSet().toList();

      lead = uniqueLeadList.map((item) => jsonDecode(item)).toList();

      print("Deal List====" + deal.toString());

      jsonFormFilterData = json.encode({
        'fields': [
          {
            'apiKeyValue': 'accountid',
            'key': 'Status',
            'type': 'select',
            'label': 'Account Id',
            'required': false,
            'value': '',
            'items': account != null
                ? account
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
          {
            'apiKeyValue': 'contactid',
            'key': 'Status',
            'type': 'select',
            'label': 'Contact',
            'required': false,
            'value': '',
            'items': contact != null
                ? contact
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
          {
            'apiKeyValue': 'leadid',
            'key': 'Status',
            'type': 'select',
            'label': 'Lead',
            'required': false,
            'value': '',
            'items': lead != null
                ? lead
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
          {
            'apiKeyValue': 'dealid',
            'key': 'Status',
            'type': 'select',
            'label': 'Deal',
            'required': false,
            'value': '',
            'items': deal != null
                ? deal
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
          {
            'apiKeyValue': 'activitytypeid',
            'key': 'Status',
            'type': 'select',
            'label': 'Activity Type',
            'required': false,
            'value': '',
            'items': activityType != null
                ? activityType
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
          {
            'apiKeyValue': 'statusid',
            'key': 'Status',
            'type': 'select',
            'label': 'Status',
            'required': false,
            'value': '',
            'items': status != null
                ? status
                : [
                    {'label': 'New', 'value': '1'},
                    {'label': 'Active', 'value': '2'},
                    {'label': 'Inactive', 'value': '3'},
                  ]
          },
        ],
      });

      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> callFilterAPI(String obj) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_TASK_URL;
    var objToSend = {
      "\"condition\"": [obj],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    TaskDetailBean contactDetailsResponseBean = TaskDetailBean.fromJson(map);

    if (contactDetailsResponseBean.status) {
      taskDetailBean = contactDetailsResponseBean;
      taskListBeanData = contactDetailsResponseBean.data;

      setState(() {
        isLoader = false;
      });
    }
  }
}
