import 'dart:convert';

import 'package:bui_component/JsonFormComponent/json_schema.dart';
import 'package:example/Constants/AppColors.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Models/account_details_bean.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/contact_details_bean.dart';
import 'package:example/Models/deals_details_bean.dart';
import 'package:example/Models/details_bean.dart';
import 'package:example/Models/leads_details_bean.dart';
import 'package:example/Models/notes_details_bean.dart';
import 'package:example/Models/task_detail_bean.dart';
import 'package:example/Utils/Loader.dart';
import 'package:example/Utils/Utility.dart';
import 'package:example/Utils/logger.dart';
import 'package:example/network/api_service.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

class TaskViewDetailScreen extends StatefulWidget {
  String pageTitle;
  String email;
  String firstName;
  String type;
  Color theme;
  Color secondaryColor;
  int id;
  Function() onCallBack;

  TaskViewDetailScreen(
      {this.pageTitle,
      this.email,
      this.firstName,
      this.type,
      this.theme,
      this.secondaryColor,
      this.id,
      this.onCallBack,
      this.taskListBeanData});

  TaskDetailList taskListBeanData;

  @override
  _TaskViewDetailScreenState createState() => _TaskViewDetailScreenState(
      pageTitle: pageTitle,
      email: email,
      firstName: firstName,
      id: id,
      widgetTaskListBeanData: taskListBeanData);
}

class _TaskViewDetailScreenState extends State<TaskViewDetailScreen> {
  final GlobalKey<ScaffoldState> _scaffoldKey = new GlobalKey<ScaffoldState>();

  bool isAccountSelected = false;
  bool isContactSelected = false;
  bool isDealSelected = false;
  bool isLeadSelected = false;
  bool isStatusSelected = false;

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    initSharedPref();
    contactScreenViewDetailsAPI();

    if (widget.type != "Add") {
      taskAPI();
      notesAPI();
    }
  }

  String ownerId;

  String pageTitle;
  String email;
  String firstName;
  int id;
  TaskDetailList widgetTaskListBeanData;

  dynamic formGeneral;

  List<dynamic> account = new List();
  List<dynamic> status = new List();
  List<dynamic> contact = new List();
  List<dynamic> lead = new List();
  List<dynamic> activityType = new List();
  List<dynamic> deal = new List();

  _TaskViewDetailScreenState(
      {this.pageTitle,
      this.email,
      this.firstName,
      this.id,
      this.widgetTaskListBeanData});

  String name = "Test";
  String form;

  dynamic response;

  Color theme = AppColors.APP_COLOR_PRIMARY;

  bool isLoader = true;
  bool isLoading = true;

  SharedPreferences prefs;

  Map decorations;

  Map validations = {'text1': validationExample};
  Map errorText = {
    'input1': "Error Value",
    'text1': "Enter the Password",
  };

  List<String> taskName = ["Open", "Close"];

  bool isExpand = false;

  TaskDetailBean taskDetailBean;

  List<TaskDetailList> taskListBeanData = new List();
  NotesDetailsBean notesDetailsBean;
  List<NotesListDetailsBean> notesListDetailsBean = new List();

  Future<void> initSharedPref() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    String ownerIdValue = prefs.getString(AppStrings.OWNER_ID);
    setState(() {
      ownerId = ownerIdValue;
    });
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
      child: Scaffold(
        key: _scaffoldKey,
        appBar: widget.type == "Add"
            ? AppBar(
                title: Text("Add Task"),
                backgroundColor: widget.theme,
              )
            : AppBar(
                title: Text("Task Info"),
                backgroundColor: widget.theme,
              ),
        body: isLoading
            ? Center(
                child: Loader(
                loaderColor: widget.theme,
              ))
            : Container(child: _formView),
      ),
    );
  }

  Widget get _formView {
    return new ListView(
      children: [
        Form(
          child: new Center(
            child: new Column(children: <Widget>[
              new JsonSchema(
                primaryColor: widget.theme ?? AppColors.primaryColor,
                secondaryColor:
                    widget.secondaryColor ?? AppColors.APP_COLOR_PRIMARY,
                decorations: decorations,
                form: form,
                validations: validations,
                errorMessages: errorText,
                buttonText: widget.type == AppStrings.ADD
                    ? AppStrings.ADD
                    : AppStrings.SAVE,
                onChanged: (dynamic response) {
                  setState(() {
                    this.response = response;
                  });

                  if (widget.type == "Add") {
                    setState(() {
                      isAccountSelected = true;
                    });
                  }
                  print("Response====" + response.toString());

                  var jsonObj = {
                    for (int i = 0; i < response['fields'].length; i++)
                      if (response['fields'][i]['items'].toString() != null)
                        json.encode(
                            response['fields'][i]['apiKeyValue']
                                .toString()): json
                            .encode(response['fields'][i]['items'].toString())
                  };
                  print("Response====" + jsonObj.toString());
                },
                actionSave: (data) {
                  Utils.hideKeyboard(context);
                  setState(() {
                    formGeneral = data;
                  });

                  var jsonObj;
                  if (widget.type != "Add") {
                    jsonObj = {
                      for (int i = 0; i < data['fields'].length; i++)
                        if (data['fields'][i]['value'].toString() != "")
                          json.encode(
                                  data['fields'][i]['apiKeyValue'].toString()):
                              json.encode(data['fields'][i]['value'].toString())
                    };
                  } else {
                    jsonObj = {
                      for (int i = 0; i < data['fields'].length; i++)
                        json.encode(
                                data['fields'][i]['apiKeyValue'].toString()):
                            json.encode(data['fields'][i]['value'].toString()),
                      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
                      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
                      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
                    };
                  }
                  Logger.logMessage(Log.DEBUG, jsonObj);

                  if (widget.type != "Add") {
                    updateTaskAPI(jsonObj.toString());
                  } else {
                    //print("JsonObj=====" + jsonObj.toString());
                    addTaskNewAPI(jsonObj.toString());
                  }
                },
                actionCancel: () {
                  Navigator.pop(context);
                },
                buttonSave: true,
              ),
            ]),
          ),
        ),
      ],
    );
  }

  static String validationExample(field, value) {
    String pattern =
        r'^(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[0-9])(?=.*?[!@#\$&*~]).{8,}$';
    RegExp regExp = new RegExp(pattern);

    if (value.isEmpty) {
      return 'Please enter some text';
    }
    if (value.length < 8) {
      return 'Password is too short';
    }

    if (!regExp.hasMatch(value)) {
      return 'Password contains Minimum 1 Upper case Minimum 1 lowercase Minimum 1 Numeric Number Minimum 1 Special Character Common Allow Character ( ! @ # \$ & * ~ )';
    }
    return null;
  }

  Future<void> updateTaskAPI(String jsonString) async {
    String urlController = ServerUrl.CRM_UPDATE_TASK_URL;
    var objToSend = {
      "\"condition\"": [
        {"\"id\"": id.toString()}
      ],
      "\"data\"": [jsonString.toString()],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      Utils.showInSnackBar(
          scaffoldKey: _scaffoldKey, message: baseBean.message);
      widget.onCallBack();
      Navigator.pop(context);
    } else {
      Utils.showInSnackBar(
          scaffoldKey: _scaffoldKey, message: baseBean.message);
    }
  }

  Future<void> accountScreenViewDetailsAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_ACCOUNT_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    AccountDetailsBean accountDetailsResponseBean =
        AccountDetailsBean.fromJson(map);

    if (accountDetailsResponseBean.status) {
      account = [
        for (int i = 0; i < accountDetailsResponseBean.data.length; i++)
          {
            'label': accountDetailsResponseBean.data[i].companyname,
            'value': accountDetailsResponseBean.data[i].id.toString(),
          },
      ];

      contactScreenViewDetailsAPI();

      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> contactScreenViewDetailsAPI() async {
    contact.clear();
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_CONTACT_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    ContactDetailsBean contactDetailsResponseBean =
        ContactDetailsBean.fromJson(map);

    if (contactDetailsResponseBean.status) {
      contact = [
        for (int i = 0; i < contactDetailsResponseBean.data.length; i++)
          {
            'label': contactDetailsResponseBean.data[i].firstname +
                " " +
                contactDetailsResponseBean.data[i].secondName,
            'value': contactDetailsResponseBean.data[i].id.toString(),
          },
      ];

      account = [
        for (int i = 0; i < contactDetailsResponseBean.data.length; i++)
          if (contactDetailsResponseBean.data[i].accountid != null)
            {
              'label': contactDetailsResponseBean.data[i].accountid.companyname
                  .toString(),
              'value':
                  contactDetailsResponseBean.data[i].accountid.id.toString(),
            },
      ];

      final jsonList1 = account.map((item) => jsonEncode(item)).toList();

      final uniqueJsonList1 = jsonList1.toSet().toList();

      account = uniqueJsonList1.map((item) => jsonDecode(item)).toList();

      isLoader = true;
      account.insert(0, {
        'label': '',
        'value': '',
      });

      print("Account List====" + account.toString());

      // leadScreenViewDetailsAPI();
      taskMasterViewAPI();
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> leadScreenViewDetailsAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_LEADS_URL;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    LeadsDetailsBean leadDetailsResponseBean = LeadsDetailsBean.fromJson(map);

    if (leadDetailsResponseBean.status) {
      lead = [
        for (int i = 0; i < leadDetailsResponseBean.data.length; i++)
          {
            'label': leadDetailsResponseBean.data[i].firstname +
                " " +
                leadDetailsResponseBean.data[i].lastname,
            'value': leadDetailsResponseBean.data[i].id.toString(),
          },
      ];
      setState(() {
        isLoader = false;
      });
      dealScreenViewDetailsAPI();
    }
  }

  Future<void> dealScreenViewDetailsAPI({String obj}) async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_DEALS_URL;
    var objToSend;
    if (obj == null) {
      objToSend = {
        "\"condition\"": [],
        "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
        "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
        "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
      };
    }
    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    DealsDetailsBean dealsDetailsBean = DealsDetailsBean.fromJson(map);

    if (dealsDetailsBean.status) {
      deal = [
        for (int i = 0; i < dealsDetailsBean.data.length; i++)
          {
            'label': dealsDetailsBean.data[i].companyid.companyname.toString(),
            'value': dealsDetailsBean.data[i].id.toString(),
          },
      ];
      setState(() {
        isLoader = false;
      });
      taskMasterViewAPI();
    }
  }

  Future<void> taskMasterViewAPI() async {
    setState(() {
      isLoader = true;
      isLoading = true;
    });
    String urlController = ServerUrl.CRM_MASTER_VIEW;
    var objToSend = {
      "\"condition\"": [],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);
    DetailsBean detailsBean = DetailsBean.fromJson(map);

    if (detailsBean.statusCode == 400) {}
    if (detailsBean.status) {
      activityType = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Activity")
            if (detailsBean.data[i].key == "Type")
              {
                'label': detailsBean.data[i].name,
                'value': detailsBean.data[i].id.toString(),
              },
      ];
      status = [
        for (int i = 0; i < detailsBean.data.length; i++)
          if (detailsBean.data[i].page == "Activity")
            if (detailsBean.data[i].key == "Status")
              {
                'label': detailsBean.data[i].name,
                'value': detailsBean.data[i].id.toString(),
              },
      ];

      if (widget.type != "Add") {
        form = json.encode({
          'fields': [
            {
              'apiKeyValue': 'contactid',
              'key': 'Status',
              'type': 'select',
              'label': 'Contact',
              'required': true,
              'value': widget.taskListBeanData.contactid.id.toString() ?? null,
              'items': contact != null
                  ? contact
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'accountid',
              'key': 'Status',
              'type': 'select',
              'label': 'Account Id',
              'required': true,
              'value': widget.taskListBeanData.accountid.id.toString() ?? null,
              'items': account != null
                  ? account
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'desc',
              'key': 'No of Employees',
              'type': 'input',
              'label': 'Description',
              'required': true,
              'hint': 'Description',
              'value': widgetTaskListBeanData.desc.toString() != "null"
                  ? widgetTaskListBeanData.desc.toString()
                  : "",
            },
            {
              'apiKeyValue': 'leadid',
              'key': 'Status',
              'type': 'select',
              'label': 'Lead',
              'required': true,
              'value': widget.taskListBeanData.leadid.id.toString() ?? null,
              'items': lead != null
                  ? lead
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'activitytypeid',
              'key': 'Status',
              'type': 'select',
              'label': 'Activity Type',
              'required': true,
              'value':
                  widget.taskListBeanData.activitytypeid.id.toString() ?? null,
              'items': activityType != null
                  ? activityType
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'dealid',
              'key': 'Status',
              'type': 'select',
              'label': 'Deal',
              'required': true,
              'value': widget.taskListBeanData.dealid.id.toString() ?? null,
              'items': deal != null
                  ? deal
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'duedate',
              'key': 'dueDate',
              'type': 'date',
              'label': 'Due Date',
              'required': true,
              'value': widgetTaskListBeanData.duedate.toString() != "null"
                  ? widgetTaskListBeanData.duedate.toString()
                  : "",
            },
            {
              'apiKeyValue': 'statusid',
              'key': 'Status',
              'type': 'select',
              'label': 'Status',
              'required': true,
              'value': widget.taskListBeanData.statusid.id.toString() ?? null,
              'items': status != null
                  ? status
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
          ]
        });
      } else {
        form = json.encode({
          'fields': [
            {
              'apiKeyValue': 'owner_id',
              'key': 'input',
              'type': 'input',
              'label': 'Owner ID',
              'required': true,
              'readOnly': true,
              'value': ownerId ?? "",
              'visibility': false,
            },
            {
              'apiKeyValue': 'contactid',
              'key': 'Status',
              'type': 'select',
              'label': 'Contact',
              'required': true,
              'value': firstName ?? null,
              'items': contact != null
                  ? contact
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'accountid',
              'key': 'Status',
              'type': 'select',
              'label': 'Account Id',
              'required': true,
              'value': firstName ?? null,
              'items': account != null
                  ? account
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            /*

            {
              'apiKeyValue': 'desc',
              'key': 'No of Employees',
              'type': 'input',
              'label': 'Description',
              'required': true,
              'hint': 'Description',
              'value': "",
            },
            {
              'apiKeyValue': 'leadid',
              'key': 'Status',
              'type': 'select',
              'label': 'Lead',
              'required': true,
              'value': firstName ?? null,
              'items': lead != null
                  ? lead
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'activitytypeid',
              'key': 'Status',
              'type': 'select',
              'label': 'Activity Type',
              'required': true,
              'value': firstName ?? null,
              'items': activityType != null
                  ? activityType
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'dealid',
              'key': 'Status',
              'type': 'select',
              'label': 'Deal',
              'required': true,
              'value': firstName ?? null,
              'items': deal != null
                  ? deal
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },
            {
              'apiKeyValue': 'duedate',
              'key': 'dueDate',
              'type': 'date',
              'label': 'Due Date',
              'required': true,
              'value': "",
            },
            {
              'apiKeyValue': 'statusid',
              'key': 'Status',
              'type': 'select',
              'label': 'Status',
              'required': true,
              'value': firstName ?? null,
              'items': status != null
                  ? status
                  : [
                      {'label': 'New', 'value': '1'},
                      {'label': 'Active', 'value': '2'},
                      {'label': 'Inactive', 'value': '3'},
                    ]
            },*/
          ]
        });
      }

      decorations = {
        'input1': InputDecoration(
          fillColor: AppColors.appWhite,
          prefixIcon: Icon(
            Icons.email,
            color: widget.theme ?? AppColors.APP_COLOR_PRIMARY,
          ),
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'Company Name': InputDecoration(
          filled: true,
          fillColor: widget.type == "Add"
              ? AppColors.APP_COLOR_WHITE
              : AppColors.BACKGROUND_GREY,
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'Status': InputDecoration(
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'Industry': InputDecoration(
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'No of Employees': InputDecoration(
          prefixIcon: Icon(
            Icons.people_rounded,
            color: widget.theme ?? AppColors.APP_COLOR_PRIMARY,
          ),
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'Annual Revenue': InputDecoration(
          prefixIcon: Icon(
            Icons.money,
            color: widget.theme ?? AppColors.APP_COLOR_PRIMARY,
          ),
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'date': InputDecoration(
          prefixIcon: Icon(
            Icons.calendar_today_rounded,
            color: widget.theme ?? AppColors.APP_COLOR_PRIMARY,
          ),
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'Phone': InputDecoration(
          prefixIcon: Icon(
            Icons.phone_iphone_rounded,
            color: widget.theme ?? AppColors.APP_COLOR_PRIMARY,
          ),
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'Website': InputDecoration(
          prefixIcon: Icon(
            Icons.web_rounded,
            color: widget.theme ?? AppColors.APP_COLOR_PRIMARY,
          ),
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'Area': InputDecoration(
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'Address Line 1': InputDecoration(
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'Address Line 2': InputDecoration(
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'State': InputDecoration(
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'Pincode': InputDecoration(
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
        'Country': InputDecoration(
          focusedBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
                color: widget.theme ?? AppColors.halfWhite, width: 2),
          ),
          enabledBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.halfWhite,
              width: 2.0,
            ),
          ),
          errorBorder: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
          border: OutlineInputBorder(
            borderRadius: BorderRadius.circular(7.0),
            borderSide: BorderSide(
              color: AppColors.errorWarning,
              width: 2.0,
            ),
          ),
        ),
      };
      setState(() {
        isLoader = false;
        isLoading = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> addTaskNewAPI(String jsonString) async {
    String urlController = ServerUrl.CRM_CREATE_TASK_URL;
    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonString.toString());
    Map map = jsonDecode(res);
    BaseResponse baseBean = BaseResponse.fromJson(map);
    if (baseBean.status) {
      Utils.showInSnackBar(
          scaffoldKey: _scaffoldKey, message: baseBean.message);
      widget.onCallBack();
      Navigator.pop(context);
    } else {
      Utils.showInSnackBar(
          scaffoldKey: _scaffoldKey, message: baseBean.message);
    }
  }

  Future<void> taskAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_TASK_URL;
    var objToSend = {
      "\"condition\"": [
        {"\"id\"": widget.id.toString()}
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    TaskDetailBean taskDetailsResponseBean = TaskDetailBean.fromJson(map);

    if (taskDetailsResponseBean.status) {
      taskListBeanData = taskDetailsResponseBean.data;

      setState(() {
        isLoader = false;
      });
    }
  }

  Future<void> notesAPI() async {
    setState(() {
      isLoader = true;
    });
    String urlController = ServerUrl.CRM_NOTES_URL;
    var objToSend = {
      "\"condition\"": [
        {"\"id\"": widget.id.toString()}
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    NotesDetailsBean notesDetailsBeanResponse = NotesDetailsBean.fromJson(map);
    if (notesDetailsBeanResponse.status) {
      notesDetailsBean = notesDetailsBeanResponse;
      notesListDetailsBean = notesDetailsBeanResponse.data;
      setState(() {
        isLoader = false;
      });
    } else {
      setState(() {
        isLoader = false;
      });
    }
  }
}
