import 'dart:convert';

import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/SignUpScreenComponent.dart';
import 'package:country_code_picker/country_code_picker.dart';
import 'package:example/Constants/AppStrings.dart';
import 'package:example/Constants/ImageAssets.dart';
import 'package:example/Constants/server_url.dart';
import 'package:example/Firebase Classes/sign_in.dart';
import 'package:example/Models/base_bean.dart';
import 'package:example/Models/user_view_bean.dart';
import 'package:example/Screen/CardStyle.dart';
import 'package:example/Screen/ERP/appUpdate.dart';
import 'package:example/main.dart';
import 'package:example/network/api_service.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:toast/toast.dart';
import 'package:universal_platform/universal_platform.dart';

import 'ERP/MainWidget.dart';
import 'LoginScreen.dart';
import 'MenuScreen.dart';
import 'OtpScreen.dart';

class SignUpScreen extends StatefulWidget {
  static String tag = AppStrings.SIGN_UP_TAG;

  @override
  SignUpScreenState createState() => SignUpScreenState();
}

class SignUpScreenState extends State<SignUpScreen> {
  //controllers
  final phoneNumberController = TextEditingController();
  final emailController = TextEditingController();
  final passwordController = TextEditingController();
  final firstNameController = TextEditingController();
  final lastNameController = TextEditingController();

  UserViewBean userViewBean;
  List<UserListBean> userListBean;

  //tab bar options
  List<String> tabBarName = [
    AppStrings.SOCIAL,
    AppStrings.MOBILE,
    AppStrings.EMAIL
  ];

  //child variables
  String phoneNumber;
  String countryCode = "+91";
  String phoneNo, verificationId, smsCode;
  bool codeSent = false;
  static String user_name;
  static String user_phoneNumber;
  static String user_email;
  static bool user_emailVerified;
  static String user_imageUrl;
  static String user_token;
  static String user_uid;
  static bool existsUser;
  static SharedPreferences sharedPreferences;

  //Firebase Auth instance

  final FirebaseAuth __firebaseAuth = FirebaseAuth.instance;

  //formkey
  var _formKey = GlobalKey<FormState>();
  var _phoneGlobalKey = GlobalKey<FormState>();

  //facebook login
  // static final FacebookLogin facebookSignIn = new FacebookLogin();

  CountryCode code;
  SharedPreferences prefs;

  bool isAppUpdateAvailable = false;

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    //_onCountryChange(code);
    getStringValuesSF();
  }

  @override
  void dispose() {
    // Clean up the controller when the widget is removed from the
    // widget tree.
    phoneNumberController.dispose();
    emailController.dispose();
    passwordController.dispose();
    firstNameController.dispose();
    lastNameController.dispose();

    super.dispose();
  }

  Color theme = AppColors.APP_COLOR_PRIMARY;

  getStringValuesSF() async {
    prefs = await SharedPreferences.getInstance();
    //Return int
    String stringValue = prefs.getString(AppStrings.PRIMARY_THEME_COLOR);
    if (stringValue != null) {
      int colorCodeInt = int.parse(stringValue);

      setState(() {
        theme = Color(colorCodeInt);
        this.mounted;
      });
    }
  }

  BuildContext bContext;

  @override
  Widget build(BuildContext context) {
    bContext = context;

    if (!isAppUpdateAvailable) {
      return Scaffold(
        body: SignUpScreenComponent(
          googleSignUp: googleSignUp,
          facebookSignUp: facebookSignUp,
          phoneNumberController: phoneNumberController,
          phoneNumberSignInFunction: phoneNumberSignInFunction,
          globalKey: _formKey,
          phoneGlobalKey: _phoneGlobalKey,
          emailController: emailController,
          passwordController: passwordController,
          firstNameController: firstNameController,
          lastNameController: lastNameController,
          emailPasswordSignInFunction: emailPasswordSignInFunction,
          signInFunction: signInPage,
          tabEnabledName: tabBarName,
          themeColor: theme,
          imageLogo: Container(
            height: 120,
            width: 100,
            child: CircleAvatar(
              radius: 10,
              backgroundColor: Colors.white,
              child: Container(
                padding: EdgeInsets.all(20),
                child: Image.asset(
                  ImageAssets.LOGO_WITH_NAME,
                  height: 120,
                  width: 120,
                ),
              ),
            ),
          ),
        ),
      );
    } else {
      return Scaffold(
        body: AppForceUpdate(),
      );
    }
  }

  Future<void> googleSignUp() async {
    if (UniversalPlatform.isAndroid) {
      signInWithGoogle().then((result) {
        if (result != null) {
          user_email = email;
          user_name = name;
          user_phoneNumber = phoneNumber;
          user_emailVerified = emailVerified;
          user_imageUrl = imageUrl;
          user_token = "Bearer " + bearToken.toString();
          user_uid = uid;

          MyHomePageState.user_token = user_token;

          if (authResultData.additionalUserInfo.isNewUser) {
            userCreateAPI(bContext);
          } else {
            userViewAPI(bContext);
          }

          prefs.setString(AppStrings.EMAIL, email);

          Toast.show("Login Success" + name, context);
        } else {
          Toast.show("Login Failed", context);
        }
      }).catchError((onError) {});
    }
    else if (UniversalPlatform.isWeb) {
      await Firebase.initializeApp();
      User user;

      GoogleAuthProvider authProvider = GoogleAuthProvider();

      try {
        final UserCredential userCredential =
            await __firebaseAuth.signInWithPopup(authProvider);

        __firebaseAuth.currentUser.getIdToken().then((value) {
          MyHomePageState.user_token = "Bearer " + value;

          print("UserToken====" + MyHomePageState.user_token);

          if (userCredential.additionalUserInfo.isNewUser) {
            userCreateAPI(bContext);
          } else {
            userViewAPI(bContext);
          }
        });

        user = userCredential.user;
      } catch (e) {
        print(e);
      }

      if (user != null) {
        user_uid = user.uid;
        name = user.displayName;
        user_email = user.email;
        imageUrl = user.photoURL;
        user_token = MyHomePageState.user_token;
      }

      return user;
    }
  }

  Future<void> facebookSignUp() async {
    Toast.show("Facebook Sign in Method Calling", context);
    Navigator.of(context).push(MaterialPageRoute(
      builder: (context) {
        return CardStyle();
      },
    ));
    //facebookLogin();
  }

  Future<void> emailPasswordSignInFunction() async {
    final isValid = _formKey.currentState.validate();
    if (!isValid) {
      return;
    }
    _formKey.currentState.save();
    signUp(emailController.text.toString(), passwordController.text.toString());
  }

  Future<void> signUp(String email, String password) async {
    __firebaseAuth
        .createUserWithEmailAndPassword(email: email, password: password)
        .then((result) {
      if (result != null) {
        Navigator.of(context).push(
          MaterialPageRoute(
            builder: (context) {
              return MenuScreen();
            },
          ),
        );
        Toast.show("Login Success", context);
      } else {
        Toast.show("Login Failed", context);
      }
    });
  }

  Future<void> phoneNumberSignInFunction() async {
    final isValid = _phoneGlobalKey.currentState.validate();
    if (!isValid) {
      return;
    }
    _phoneGlobalKey.currentState.save();
    phoneNumber = phoneNumberController.text.toString();

    if (phoneNumber.isNotEmpty) {
      if (phoneNumber.length == 10) {
        //verifyPhone(phoneNumber);
        Navigator.pushReplacement(context,
            MaterialPageRoute(builder: (BuildContext context) => OtpScreen()));
      } else {}
    } else {}
  }

  Future<void> signInPage() async {
    Navigator.pushReplacement(context,
        MaterialPageRoute(builder: (BuildContext context) => LoginScreen()));
  }

  Future<void> verifyPhone(String num) async {
    final PhoneCodeAutoRetrievalTimeout autoRetrievalTimeout = (String verId) {
      this.verificationId = verId;
    };

    final PhoneCodeSent smsCodeSent = (String verId, [int forceCodeResend]) {
      this.verificationId = verId;
    };

    final PhoneVerificationCompleted verifiedSuccess =
        (PhoneAuthCredential user) {};

    final PhoneVerificationFailed verifiedFailed =
        (FirebaseAuthException user) {};

    await FirebaseAuth.instance.verifyPhoneNumber(
      phoneNumber: countryCode + " " + num,
      codeAutoRetrievalTimeout: autoRetrievalTimeout,
      codeSent: smsCodeSent,
      timeout: const Duration(seconds: 10),
      verificationCompleted: verifiedSuccess,
      verificationFailed: verifiedFailed,
    );
  }

/*  Future<void> facebookLogin() async {
    final FacebookLoginResult result = await facebookSignIn.logIn(['email']);

    switch (result.status) {
      case FacebookLoginStatus.loggedIn:
        final FacebookAccessToken accessToken = result.accessToken;

        break;
      case FacebookLoginStatus.cancelledByUser:

        break;
      case FacebookLoginStatus.error:

        break;
    }
  }

  Future<void> _logOut() async {
    await facebookSignIn.logOut();

  }*/

  Future<void> userCreateAPI(BuildContext context) async {
    String urlController = ServerUrl.CRM_USER_CREATE_API;
    var objToSend = {
      "\"uid\"": "\"${user_uid.toString()}\"",
      "\"name\"": "\"${user_name.toString()}\"",
      "\"phone\"": "\"${user_phoneNumber.toString()}\"",
      "\"email\"": "\"${user_email.toString()}\"",
      "\"photourl\"": "\"${user_imageUrl.toString()}\"",
      "\"roleid\"": "\"${1}\"",
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    BaseResponse response = BaseResponse.fromJson(map);

    if (response.status) {
      userViewAPI(context);
    }
  }

  Future<void> userViewAPI(BuildContext context) async {
    String urlController = ServerUrl.CRM_USER_API;
    var objToSend = {
      "\"condition\"": [
        {"\"uid\"": "\"${user_uid.toString()}\""}
      ],
      "\"${UrlParams.AID}\"": "\"${UrlParams.AID_VALUE}\"",
      "\"${UrlParams.AKEY}\"": "\"${UrlParams.AKEY_VALUE}\"",
      "\"${UrlParams.AV}\"": "\"${UrlParams.AV_VALUE}\"",
    };

    String jsonStr = objToSend.toString();

    String res = await APIClient.getInstance().callAPI(
        context, urlController, HTTPMethod.POST,
        isHeader: true, requestBody: jsonStr);
    Map map = jsonDecode(res);

    UserViewBean userViewBeanResponse = UserViewBean.fromJson(map);

    if (userViewBeanResponse.status == false) {}
    if (userViewBeanResponse.status) {
      setState(() {
        userViewBean = userViewBeanResponse;
        userListBean = userViewBeanResponse.data;

        prefs.setString(
            AppStrings.OWNER_ID, userViewBeanResponse.data[0].id.toString());

        Navigator.of(context).pushReplacement(
          MaterialPageRoute(
            builder: (context) {
              return MainWidget();
            },
          ),
        );
      });
      /*  Navigator.of(context).push(
          MaterialPageRoute(
            builder: (context) {
              return MainWidget();
            },
          ),
        );
      });*/

      /* Navigator.of(context).push(
          MaterialPageRoute(
            builder: (context) {
              return MenuScreen();
            },
          ),
        );*/
    }
  }
}
