import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/fontFamily.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:flutter/material.dart';

import '../functions.dart';

class SimpleDate extends StatefulWidget {
  SimpleDate({
    Key key,
    this.item,
    this.onChange,
    this.position,
    this.errorMessages = const {},
    this.validations = const {},
    this.decorations = const {},
    this.keyboardTypes = const {},
    this.primaryColor,
    this.secondaryColor,
  }) : super(key: key);
  final Color primaryColor;
  final Color secondaryColor;
  final dynamic item;
  final Function onChange;

  final int position;
  final Map errorMessages;
  final Map validations;
  final Map decorations;
  final Map keyboardTypes;

  @override
  _SimpleDate createState() => new _SimpleDate();
}

class _SimpleDate extends State<SimpleDate> {
  dynamic item;

  @override
  void initState() {
    super.initState();
    item = widget.item;
  }

  @override
  Widget build(BuildContext context) {
    Widget label = SizedBox.shrink();
    if (Fun.labelHidden(item)) {
      label = Visibility(
        visible: item['visibility'] != null ? item['visibility'] : true,
        child: new Container(
          padding: EdgeInsets.only(bottom: 5),
          child: RichText(
            text: TextSpan(
              text: item['label'],
              style: new TextStyle(
                  letterSpacing: 1,
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_16,
                  color: AppColors.APP_COLOR_BLACK,
                  fontFamily: FontFamily.Gilroy),
              children: <TextSpan>[
                //  item['label'].toString().endsWith('*')
                item['required'] == true
                    ? TextSpan(
                        text: ' *',
                        style: TextStyle(
                            letterSpacing: 1,
                            fontWeight: FontWeight.bold,
                            color: AppColors.errorWarning))
                    : TextSpan(
                        text: '',
                        style: TextStyle(
                            letterSpacing: 1,
                            fontWeight: FontWeight.bold,
                            fontSize: FontSize.FONT_13,
                            fontFamily: FontFamily.Gilroy,
                            color: AppColors.errorWarning)),
              ],
            ),
          ),
        ),
      );
    }
    return IgnorePointer(
      ignoring: item['readOnly'] != null ? item['readOnly'] : false,
      child: Visibility(
        visible: item['visibility'] != null ? item['visibility'] : true,
        child: new Container(
          margin: new EdgeInsets.only(top: 5.0),
          padding: EdgeInsets.only(bottom: 5, top: 5),
          child: new Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              label,
              new Column(
                mainAxisAlignment: MainAxisAlignment.start,
                children: <Widget>[
                  InkWell(
                    child: new TextFormField(
                      onTap: () {
                        FocusScope.of(context).requestFocus(new FocusNode());
                        selectDate();
                      },
                      readOnly: item['readOnly'] == true ||
                              item['readOnly'] == 'True' ||
                              item['readOnly'] == 'true'
                          ? true
                          : false,
                      decoration: InputDecoration(
                        filled: true,
                        fillColor: item['readOnly'] == true
                            ? AppColors.halfWhite
                            : Colors.white,
                        //border: OutlineInputBorder(),
                        enabledBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(5.0),
                          borderSide: BorderSide(
                            color: widget.secondaryColor,
                            width: 1.0,
                          ),
                        ),
                        focusedBorder: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(5.0),
                          borderSide: BorderSide(
                            color: widget.secondaryColor,
                          ),
                        ),

                        contentPadding: const EdgeInsets.all(8.0),
                        border: OutlineInputBorder(
                          borderRadius: BorderRadius.circular(5.0),
                        ),
                        hintText: item['value'] ?? "",
                        //prefixIcon: Icon(Icons.date_range_rounded),
                        suffixIcon: IconButton(
                          onPressed: () {
                            //selectDate();
                          },
                          icon: Icon(
                            Icons.calendar_today_rounded,
                            color: widget.secondaryColor,
                          ),
                        ),
                      ),
                      validator: (String value) {
                        return null;
                      },
                    ),
                  ),
                ],
              )
            ],
          ),
        ),
      ),
    );
  }

  Future selectDate() async {
    DateTime picked = await showDatePicker(
      context: context,
      initialDate: new DateTime.now().subtract(new Duration(days: 360)),
      firstDate: new DateTime.now().subtract(new Duration(days: 360)),
      lastDate: DateTime(2100),
      builder: (BuildContext context, Widget child) {
        return Theme(
          data: Theme.of(context).copyWith(
              colorScheme: Theme.of(context).colorScheme.copyWith(
                  primary:
                      widget.secondaryColor ?? AppColors.APP_COLOR_PRIMARY)),
          child: child,
        );
      },
    );
    if (picked != null) {
      String date =
          "${picked.year.toString()}-${picked.month.toString().padLeft(2, '0')}-${picked.day.toString().padLeft(2, '0')}";
      this.setState(() {
        item['value'] = date;
        widget.onChange(widget.position, date);
      });
    }
  }
}
