import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/fontFamily.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:flutter/material.dart';

import '../functions.dart';

class SimpleRadios extends StatefulWidget {
  SimpleRadios({
    Key key,
    this.item,
    this.onChange,
    this.position,
    this.errorMessages = const {},
    this.validations = const {},
    this.decorations = const {},
    this.keyboardTypes = const {},
    this.primaryColor,
    this.secondaryColor,
  }) : super(key: key);
  final Color primaryColor;
  final Color secondaryColor;
  final dynamic item;
  final Function onChange;

  final int position;
  final Map errorMessages;
  final Map validations;
  final Map decorations;
  final Map keyboardTypes;

  @override
  _SimpleRadios createState() => new _SimpleRadios();
}

class _SimpleRadios extends State<SimpleRadios> {
  dynamic item;
  int radioValue;

  String isRequired(item, value) {
    if (value.isEmpty) {
      return widget.errorMessages[item['key']] ?? 'Please enter some text';
    }
    return null;
  }

  @override
  void initState() {
    super.initState();
    item = widget.item;
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> radios = [];

    if (Fun.labelHidden(item)) {
      radios.add(Visibility(
        visible: item['visibility'] != null ? item['visibility'] : true,
        child: Container(
          padding: EdgeInsets.only(bottom: 5),
          child: RichText(
            text: TextSpan(
              text: item['label'],
              style: new TextStyle(
                  letterSpacing: 1,
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_16,
                  color: AppColors.APP_COLOR_BLACK,
                  fontFamily: FontFamily.Gilroy),
              children: <TextSpan>[
                //  item['label'].toString().endsWith('*')
                item['required'] == true
                    ? TextSpan(
                        text: ' *',
                        style: TextStyle(
                            letterSpacing: 1,
                            fontWeight: FontWeight.bold,
                            color: AppColors.errorWarning))
                    : TextSpan(
                        text: '',
                        style: TextStyle(
                            letterSpacing: 1,
                            fontWeight: FontWeight.bold,
                            fontSize: FontSize.FONT_13,
                            fontFamily: FontFamily.Gilroy,
                            color: AppColors.errorWarning)),
              ],
            ),
          ),
        ),
      ));
    }
    radioValue = item['value'];
    for (var i = 0; i < item['items'].length; i++) {
      radios.add(
        new Row(
          children: <Widget>[
            new Expanded(child: new Text(item['items'][i]['label'])),
            new Radio<dynamic>(
                value: item['items'][i]['value'],
                groupValue: radioValue,
                activeColor: widget.secondaryColor,
                onChanged: (dynamic value) {
                  this.setState(() {
                    radioValue = value;
                    item['value'] = value;
                    widget.onChange(widget.position, value);
                  });
                })
          ],
        ),
      );
    }
    return IgnorePointer(
      ignoring: item['readOnly'] != null ? item['readOnly'] : false,
      child: Visibility(
        visible: item['visibility'] != null ? item['visibility'] : true,
        child: new Container(
          margin: new EdgeInsets.only(top: 5.0),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: radios,
          ),
        ),
      ),
    );
  }
}
