import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/fontFamily.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:flutter/material.dart';

import '../functions.dart';

class SimpleSelect extends StatefulWidget {
  SimpleSelect({
    Key key,
    this.item,
    this.onChange,
    this.position,
    this.errorMessages = const {},
    this.validations = const {},
    this.decorations = const {},
    this.keyboardTypes = const {},
    this.primaryColor,
    this.secondaryColor,
  }) : super(key: key);
  final Color primaryColor;
  final Color secondaryColor;
  final dynamic item;
  final Function onChange;

  final int position;
  final Map errorMessages;
  final Map validations;
  final Map decorations;
  final Map keyboardTypes;

  @override
  _SimpleSelect createState() => new _SimpleSelect();
}

class _SimpleSelect extends State<SimpleSelect> {
  dynamic item;

  String isRequired(item, value) {
    if (value.isEmpty) {
      return widget.errorMessages[item['key']] ?? 'Please enter some text';
    }
    return null;
  }

  @override
  void initState() {
    super.initState();
    item = widget.item;
  }

  @override
  Widget build(BuildContext context) {
    Widget label = SizedBox.shrink();
    if (Fun.labelHidden(item)) {
      label = IgnorePointer(
        ignoring: item['readOnly'] != null ? item['readOnly'] : false,
        child: Visibility(
          visible: item['visibility'] != null ? item['visibility'] : true,
          child: Container(
            padding: EdgeInsets.only(bottom: 5),
            child: RichText(
              text: TextSpan(
                text: item['label'],
                style: new TextStyle(
                    letterSpacing: 1,
                    fontWeight: FontWeight.bold,
                    fontSize: FontSize.FONT_16,
                    color: AppColors.APP_COLOR_BLACK,
                    fontFamily: FontFamily.Gilroy),
                children: <TextSpan>[
                  //  item['label'].toString().endsWith('*')
                  item['required'] == true
                      ? TextSpan(
                          text: ' *',
                          style: TextStyle(
                              letterSpacing: 1,
                              fontWeight: FontWeight.bold,
                              color: AppColors.errorWarning))
                      : TextSpan(
                          text: '',
                          style: TextStyle(
                              letterSpacing: 1,
                              fontWeight: FontWeight.bold,
                              fontSize: FontSize.FONT_13,
                              fontFamily: FontFamily.Gilroy,
                              color: AppColors.errorWarning)),
                ],
              ),
            ),
          ),
        ),
      );
    }
    return IgnorePointer(
      ignoring: item['readOnly'] != null ? item['readOnly'] : false,
      child: Visibility(
        visible: item['visibility'] ?? true,
        child: new Container(
          margin: new EdgeInsets.only(top: 5.0, bottom: 5),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: <Widget>[
              label,
              Container(
                padding: EdgeInsets.only(left: 5),
                decoration: ShapeDecoration(
                  color: item['readOnly'] == true
                      ? AppColors.halfWhite
                      : Colors.white,
                  shape: RoundedRectangleBorder(
                    side: BorderSide(
                        width: 1.0,
                        style: BorderStyle.solid,
                        color: widget.secondaryColor),
                    borderRadius: BorderRadius.all(
                      Radius.circular(5.0),
                    ),
                  ),
                ),
                child: new DropdownButton<String>(
                  isExpanded: true,
                  hint: new Text("Choose something"),
                  value: item['value'],
                  //  dropdownColor: Colors.red,

                  onChanged: (String newValue) {
                    setState(() {
                      item['value'] = newValue;

                      widget.onChange(widget.position, newValue);
                    });
                  },
                  icon: Icon(
                    // Add this
                    Icons.arrow_drop_down, // Add this
                    color: widget.secondaryColor, // Add this
                  ),
                  underline: Container(),

                  items: item['items']
                      .map<DropdownMenuItem<String>>((dynamic data) {
                    return DropdownMenuItem<String>(
                      value: data['value'],
                      child: new Text(
                        data['label'] ?? "",
                        style: new TextStyle(color: Colors.black),
                      ),
                    );
                  }).toList(),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
