import 'package:bui_component/Constants/AppColors.dart';
import 'package:bui_component/Constants/fontFamily.dart';
import 'package:bui_component/Constants/fontSize.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

import '../functions.dart';

class SimpleText extends StatefulWidget {
  SimpleText({
    Key key,
    this.item,
    this.onChange,
    this.position,
    this.errorMessages = const {},
    this.validations = const {},
    this.decorations = const {},
    this.keyboardTypes = const {},
    this.primaryColor,
    this.secondaryColor,
  }) : super(key: key);
  final Color primaryColor;
  final Color secondaryColor;
  final dynamic item;
  final Function onChange;

  final int position;
  final Map errorMessages;
  final Map validations;
  final Map decorations;
  final Map keyboardTypes;

  @override
  _SimpleText createState() => new _SimpleText();
}

class _SimpleText extends State<SimpleText> {
  dynamic item;

  String isRequired(item, value) {
    if (value.isEmpty) {
      return widget.errorMessages[item['key']] ?? 'Please enter some text';
    }
    return null;
  }

  @override
  void initState() {
    // TODO: implement initState
    super.initState();
    item = widget.item;
  }

  @override
  Widget build(BuildContext context) {
    Widget label = SizedBox.shrink();
    if (Fun.labelHidden(item)) {
      label = Visibility(
        visible: item['visibility'] != null ? item['visibility'] : true,
        child: new Container(
          padding: EdgeInsets.only(bottom: 5),
          child: RichText(
            text: TextSpan(
              text: item['label'],
              style: new TextStyle(
                  letterSpacing: 1,
                  fontWeight: FontWeight.bold,
                  fontSize: FontSize.FONT_16,
                  color: AppColors.APP_COLOR_BLACK,
                  fontFamily: FontFamily.Gilroy),
              children: <TextSpan>[
                //  item['label'].toString().endsWith('*')
                item['required'] == true
                    ? TextSpan(
                        text: ' *',
                        style: TextStyle(
                            letterSpacing: 1,
                            fontWeight: FontWeight.bold,
                            color: AppColors.errorWarning))
                    : TextSpan(
                        text: '',
                        style: TextStyle(
                            letterSpacing: 1,
                            fontWeight: FontWeight.bold,
                            fontSize: FontSize.FONT_13,
                            fontFamily: FontFamily.Gilroy,
                            color: AppColors.errorWarning)),
              ],
            ),
          ),
        ),
      );
    }
    return Visibility(
      visible: item['visibility'] != null ? item['visibility'] : true,
      child: new Container(
        margin: new EdgeInsets.only(top: 5.0),
        child: new Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            label,
            new TextFormField(
              controller: null,
              initialValue: item['value'] ?? null,
              decoration: item['decoration'] ??
                  widget.decorations[item['key']] ??
                  new InputDecoration(
                    filled: true,
                    fillColor: item['readOnly'] == true
                        ? AppColors.halfWhite
                        : Colors.white,
                    hintText: item['placeholder'] ?? "",
                    helperText: item['helpText'] ?? "",
                    enabledBorder: OutlineInputBorder(
                      borderRadius: BorderRadius.circular(5.0),
                      borderSide: BorderSide(
                        color: widget.secondaryColor,
                        width: 1.0,
                      ),
                    ),
                    focusedBorder: OutlineInputBorder(
                      borderRadius: BorderRadius.circular(5.0),
                      borderSide: BorderSide(
                        color: widget.secondaryColor,
                      ),
                    ),
                    contentPadding: const EdgeInsets.all(8.0),
                    border: OutlineInputBorder(
                      borderRadius: BorderRadius.circular(5.0),
                    ),
                  ),
              maxLines: item['type'] == "textArea" ? 10 : 1,
              maxLength: item['type'] == "textArea"
                  ? item['textLength'] ?? 1000
                  : null,
              onChanged: (String value) {
                item['value'] = value;
                // _handleChanged();

                widget.onChange(widget.position, value);
              },
              obscureText: item['type'] == "password" ? true : false,
              readOnly: item['readOnly'] == true ||
                      item['readOnly'] == 'True' ||
                      item['readOnly'] == 'true'
                  ? true
                  : false,
              keyboardType: item['keyboardType'] ??
                  widget.keyboardTypes[item['key']] ??
                  TextInputType.text,
              validator: (value) {
                if (widget.validations.containsKey(item['key'])) {
                  return widget.validations[item['key']](item, value);
                }
                if (item.containsKey('validator')) {
                  if (item['validator'] != null) {
                    if (item['validator'] is Function) {
                      return item['validator'](item, value);
                    }
                  }
                }
                if (item['type'] == "email") {
                  return Fun.validateEmail(item, value);
                }

                if (item.containsKey('required')) {
                  if (item['required'] == true ||
                      item['required'] == 'True' ||
                      item['required'] == 'true') {
                    return isRequired(item, value);
                  }
                }
                return null;
              },
            ),
          ],
        ),
      ),
    );
  }
}
