/*
    Refer License for Terms & Conditions
 */

import 'dart:convert';

import 'package:bui_component/ButtonsComponent/buttonComponent.dart';
import 'package:bui_component/Constants/AppColors.dart';
import 'package:flutter/material.dart';

import 'components/index.dart';
import 'components/simple_date.dart';
import 'components/simple_radios.dart';

class JsonSchema extends StatefulWidget {
  const JsonSchema({
    @required this.form,
    @required this.onChanged,
    this.padding,
    this.formMap,
    this.autovalidateMode,
    this.errorMessages = const {},
    this.validations = const {},
    this.decorations = const {},
    this.keyboardTypes = const {},
    this.buttonSave,
    this.buttonCancel,
    this.actionSave,
    this.actionCancel,
    this.secondaryColor,
    this.primaryColor,
    this.buttonText,
  });

  final Color secondaryColor;
  final Color primaryColor;
  final String buttonText;
  final Map errorMessages;
  final Map validations;
  final Map decorations;
  final Map keyboardTypes;
  final String form;
  final Map formMap;
  final double padding;
  final bool buttonSave;
  final bool buttonCancel;
  final Function actionSave;
  final Function actionCancel;
  final ValueChanged<dynamic> onChanged;
  final AutovalidateMode autovalidateMode;

  @override
  _CoreFormState createState() =>
      new _CoreFormState(formMap ?? json.decode(form));
}

class _CoreFormState extends State<JsonSchema> {
  final dynamic formGeneral;

  int radioValue;

  List<Widget> jsonToForm() {
    List<Widget> listWidget = new List<Widget>();
    if (formGeneral['title'] != null) {
      listWidget.add(Text(
        formGeneral['title'],
        style: new TextStyle(fontWeight: FontWeight.bold, fontSize: 20.0),
      ));
    }
    if (formGeneral['description'] != null) {
      listWidget.add(Text(
        formGeneral['description'],
        style: new TextStyle(fontSize: 14.0, fontStyle: FontStyle.italic),
      ));
    }

    for (var count = 0; count < formGeneral['fields'].length; count++) {
      Map item = formGeneral['fields'][count];

      if (item['type'] == "input" ||
          item['type'] == "password" ||
          item['type'] == "email" ||
          item['type'] == "textArea" ||
          item['type'] == "textInput") {
        listWidget.add(new SimpleText(
          item: item,
          onChange: onChange,
          position: count,
          decorations: widget.decorations,
          errorMessages: widget.errorMessages,
          validations: widget.validations,
          keyboardTypes: widget.keyboardTypes,
          primaryColor: widget.primaryColor ?? AppColors.primaryColor,
          secondaryColor: widget.secondaryColor ?? AppColors.REDDISH_GREY,
        ));
      }

      if (item['type'] == "radioButton") {
        listWidget.add(new SimpleRadios(
          item: item,
          onChange: onChange,
          position: count,
          decorations: widget.decorations,
          errorMessages: widget.errorMessages,
          validations: widget.validations,
          keyboardTypes: widget.keyboardTypes,
          primaryColor: widget.primaryColor ?? AppColors.primaryColor,
          secondaryColor: widget.secondaryColor ?? AppColors.REDDISH_GREY,
        ));
      }

      if (item['type'] == "switch") {
        listWidget.add(new SimpleSwitch(
          item: item,
          onChange: onChange,
          position: count,
          decorations: widget.decorations,
          errorMessages: widget.errorMessages,
          validations: widget.validations,
          keyboardTypes: widget.keyboardTypes,
          primaryColor: widget.primaryColor ?? AppColors.primaryColor,
          secondaryColor: widget.secondaryColor ?? AppColors.REDDISH_GREY,
        ));
      }

      if (item['type'] == "checkbox") {
        listWidget.add(new SimpleListCheckbox(
          item: item,
          onChange: onChange,
          position: count,
          decorations: widget.decorations,
          errorMessages: widget.errorMessages,
          validations: widget.validations,
          keyboardTypes: widget.keyboardTypes,
          primaryColor: widget.primaryColor ?? AppColors.primaryColor,
          secondaryColor: widget.secondaryColor ?? AppColors.REDDISH_GREY,
        ));
      }

      if (item['type'] == "select") {
        listWidget.add(new SimpleSelect(
          item: item,
          onChange: onChange,
          position: count,
          decorations: widget.decorations,
          errorMessages: widget.errorMessages,
          validations: widget.validations,
          keyboardTypes: widget.keyboardTypes,
          primaryColor: widget.primaryColor ?? AppColors.primaryColor,
          secondaryColor: widget.secondaryColor ?? AppColors.REDDISH_GREY,
        ));
      }

      if (item['type'] == "date") {
        listWidget.add(new SimpleDate(
          item: item,
          onChange: onChange,
          position: count,
          decorations: widget.decorations,
          errorMessages: widget.errorMessages,
          validations: widget.validations,
          keyboardTypes: widget.keyboardTypes,
          primaryColor: widget.primaryColor ?? AppColors.primaryColor,
          secondaryColor: widget.secondaryColor ?? AppColors.REDDISH_GREY,
        ));
      }
    }

    if (widget.buttonSave) {
      listWidget.add(Container(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.spaceAround,
          children: [
            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Button.mini(
                width: MediaQuery.of(context).size.width * 0.3,
                onPressed: () {
                  widget.actionCancel();
                },
                buttonSize: ButtonSize.medium,
                buttonType: ButtonType.cancel,
                btnColor: AppColors.APP_BUTTON_GREY,
                btnTextColor:
                    widget.secondaryColor ?? AppColors.APP_COLOR_PRIMARY,
                elevation: 10,
              ),
            ),
            Padding(
              padding: const EdgeInsets.all(8.0),
              child: Button.mini(
                width: MediaQuery.of(context).size.width * 0.3,
                onPressed: () {
                  if (_formKey.currentState.validate()) {
                    widget.actionSave(formGeneral);
                  }
                },
                buttonSize: ButtonSize.medium,
                buttonType: ButtonType.custom,
                btnColor: widget.secondaryColor ?? AppColors.APP_COLOR_PRIMARY,
                btnTextColor: Colors.white,
                btnText: widget.buttonText ?? "Save",
                elevation: 10,
              ),
            ),
          ],
        ),
      ));
    }

    return listWidget;
  }

  _CoreFormState(this.formGeneral);

  void _handleChanged() {
    widget.onChanged(formGeneral);
  }

  void onChange(int position, dynamic value) {
    this.setState(() {
      formGeneral['fields'][position]['value'] = value;
      this._handleChanged();
    });
  }

  final _formKey = GlobalKey<FormState>();

  @override
  Widget build(BuildContext context) {
    return Form(
      autovalidateMode:
          formGeneral['autoValidated'] ?? AutovalidateMode.disabled,
      key: _formKey,
      child: new Container(
        padding: new EdgeInsets.all(widget.padding ?? 8.0),
        child: new Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: jsonToForm(),
        ),
      ),
    );
  }
}
