import 'package:flutter/material.dart';
import 'package:intl/intl.dart';

class BuiFormatter {
  String timeOfDayToJM(TimeOfDay tod) {
    final now = new DateTime.now();
    final dt = DateTime(now.year, now.month, now.day, tod.hour, tod.minute);
    final format = DateFormat.jm();

    ///"6:00 AM"
    return format.format(dt);
  }

  String timeOfDayToDateTimeString(TimeOfDay tod) {
    final now = new DateTime.now();
    final dt = DateTime(now.year, now.month, now.day, tod.hour, tod.minute);
    return dt.toString();
  }

  TimeOfDay dateTimeNowToTimeOfDay(String date) {
    final now = DateTime.parse(date);
    final dt = DateTime(now.year, now.month, now.day, now.hour, now.minute);
    return TimeOfDay.fromDateTime(dt);
  }

  TimeOfDay dateTimeToTimeOfDay(DateTime date) {
    final dt =
        DateTime(date.year, date.month, date.day, date.hour, date.minute);
    return TimeOfDay.fromDateTime(dt);
  }

  String dateTimeToStringDate(DateTime date) {
    final format = DateFormat.jm();
    String time = format.format(date);
    String resdDate = DateFormat('dd-MM-yyyy').format(date);
    return "$time - $resdDate";
  }

  String dateTimeToStringDateReverse(DateTime date) {
    final format = DateFormat.jm();
    String time = format.format(date);
    String resdDate = DateFormat('yyyy-MM-dd').format(date);
    return "$time - $resdDate";
  }

  String dateToStringDate(DateTime date) {
    String resdDate = DateFormat('dd-MM-yyyy').format(date);
    return "$resdDate";
  }

  String dateToStringDateReverse(DateTime date) {
    String resdDate = DateFormat('yyyy-MM-dd').format(date);
    return "$resdDate";
  }

  String buiDateTimeDDMMYYFormat(String date) {
    DateTime dateParse = DateTime.parse(date.toString());
    String dateFormat = DateFormat('dd-MM-yyyy').format(dateParse);
    return dateFormat;
  }

  String buiDateTimeYYYYMMDDFormat(String date) {
    DateTime dateParse = DateTime.parse(date.toString());
    String dateFormat = DateFormat('yyyy-MM-dd').format(dateParse);
    return dateFormat;
  }

  DateTime firestoreTimeToDateTime(firebasDate) {
    return DateTime.parse(firebasDate.toDate().toString());
  }

  List firestoreTimeToMonthYear(firebasDate) {
    DateTime parsed = DateTime.parse(firebasDate.toDate().toString());
    String year = DateFormat('yyyy').format(parsed);
    String month = DateFormat('MM').format(parsed);
    String day = DateFormat('dd').format(parsed);
    return [year, month, day];
  }

  int getHourAndMinute(String dbDate) {
    DateTime parsed = DateTime.parse(dbDate);
    String hour = DateFormat('H').format(parsed);
    String minute = DateFormat('mm').format(parsed);
    String joined = "$hour$minute";
    return int.parse(joined);
  }

  String getMonthName(int monthNumber) {
    switch (monthNumber) {
      case 1:
        return "January";
        break;
      case 2:
        return "February";
        break;
      case 3:
        return "March";
        break;
      case 4:
        return "April";
        break;
      case 5:
        return "May";
        break;
      case 6:
        return "June";
        break;
      case 7:
        return "July";
        break;
      case 8:
        return "August";
        break;
      case 9:
        return "September";
        break;
      case 10:
        return "October";
        break;
      case 11:
        return "November";
        break;
      case 12:
        return "December";
        break;
      default:
        return "Unknown";
    }
  }

  String prevYear(currentYear) {
    int curYr;
    if (currentYear is String) {
      curYr = int.parse(currentYear);
    } else {
      curYr = currentYear;
    }
    int finalYr = curYr - 1;
    return finalYr.toString();
  }

  String getPrevMonth(int currentMonth) {
    int finalMnth = currentMonth - 1;
    if (finalMnth < 10) {
      return "0$finalMnth";
    } else {
      return finalMnth.toString();
    }
  }

  int getPreviousMonthNum(int currentMonth) {
    if (currentMonth == 1) {
      return 12;
    } else {
      return currentMonth - 1;
    }
  }

  String getCurrentMonthName() {
    DateTime now = DateTime.now();
    String month = DateFormat('MM').format(now);
    int monthNumber = int.parse(month);
    switch (monthNumber) {
      case 1:
        return "January";
        break;
      case 2:
        return "February";
        break;
      case 3:
        return "March";
        break;
      case 4:
        return "April";
        break;
      case 5:
        return "May";
        break;
      case 6:
        return "June";
        break;
      case 7:
        return "July";
        break;
      case 8:
        return "August";
        break;
      case 9:
        return "September";
        break;
      case 10:
        return "October";
        break;
      case 11:
        return "November";
        break;
      case 12:
        return "December";
        break;
      default:
        return "Unknown";
    }
  }

  String getPreviousMonthName(int currentMonth) {
    int monthNumber = currentMonth - 1;

    switch (monthNumber) {
      case 0:
        return "December";
        break;
      case 1:
        return "January";
        break;
      case 2:
        return "February";
        break;
      case 3:
        return "March";
        break;
      case 4:
        return "April";
        break;
      case 5:
        return "May";
        break;
      case 6:
        return "June";
        break;
      case 7:
        return "July";
        break;
      case 8:
        return "August";
        break;
      case 9:
        return "September";
        break;
      case 10:
        return "October";
        break;
      case 11:
        return "November";
        break;
      case 12:
        return "December";
        break;
      default:
        return "Unknown";
    }
  }

  String getPreviousMonthNameAndYear(int currentMonth) {
    DateTime now = DateTime.now();
    int monthNumber = currentMonth - 1;
    int finalYr;
    if (monthNumber == 0) {
      finalYr = now.year - 1;
    } else {
      finalYr = now.year;
    }
    switch (monthNumber) {
      case 0:
        return "December " + finalYr.toString();
        break;
      case 1:
        return "January " + finalYr.toString();
        break;
      case 2:
        return "February " + finalYr.toString();
        break;
      case 3:
        return "March " + finalYr.toString();
        break;
      case 4:
        return "April " + finalYr.toString();
        break;
      case 5:
        return "May " + finalYr.toString();
        break;
      case 6:
        return "June " + finalYr.toString();
        break;
      case 7:
        return "July " + finalYr.toString();
        break;
      case 8:
        return "August " + finalYr.toString();
        break;
      case 9:
        return "September " + finalYr.toString();
        break;
      case 10:
        return "October " + finalYr.toString();
        break;
      case 11:
        return "November " + finalYr.toString();
        break;
      default:
        return "NaN " + finalYr.toString();
    }
  }

  String getCurrencyNameByCountry(String country) {
    switch (country) {
      case "In":
        return "₹";
        break;
      case "USA":
        return "\$";
        break;
      case "Uk":
        return "£";
        break;
      default:
        return "Unknown";
    }
  }

  String buiNumberFormatConversion(num) {
    if (num > 999 && num < 99999) {
      return "${(num / 1000).toStringAsFixed(1)} K";
    } else if (num > 99999 && num < 999999) {
      return "${(num / 1000).toStringAsFixed(0)} K";
    } else if (num > 999999 && num < 999999999) {
      return "${(num / 1000000).toStringAsFixed(1)} M";
    } else if (num > 999999999) {
      return "${(num / 1000000000).toStringAsFixed(1)} B";
    } else {
      return num.toString();
    }
  }

  String buiNumberFormatConversionWithCountryCode(num, country) {
    String countryCode = country.toLowerCase().toString();
    if (countryCode == "us") {
      if (num > 999 && num < 99999) {
        return "\$ ${(num / 1000).toStringAsFixed(1)} K";
      } else if (num > 99999 && num < 999999) {
        return "\$ ${(num / 1000).toStringAsFixed(0)} K";
      } else if (num > 999999 && num < 999999999) {
        return "\$ ${(num / 1000000).toStringAsFixed(1)} M";
      } else if (num > 999999999) {
        return "\$ ${(num / 1000000000).toStringAsFixed(1)} B";
      } else {
        return "\$ " + num.toString();
      }
    } else if (countryCode == "in") {
      if (num > 999 && num < 99999) {
        return "₹ ${(num / 1000).toStringAsFixed(1)} K";
      } else if (num > 99999 && num < 999999) {
        return "₹ ${(num / 1000).toStringAsFixed(0)} K";
      } else if (num > 999999 && num < 999999999) {
        return "₹ ${(num / 1000000).toStringAsFixed(1)} L";
      } else if (num > 999999999) {
        return "₹ ${(num / 1000000000).toStringAsFixed(1)} C";
      } else {
        return "₹" + num.toString();
      }
    } else if (countryCode == "ja" || countryCode == "ch") {
      if (num > 999 && num < 99999) {
        return "¥ ${(num / 1000).toStringAsFixed(1)} K";
      } else if (num > 99999 && num < 999999) {
        return "¥ ${(num / 1000).toStringAsFixed(0)} K";
      } else if (num > 999999 && num < 999999999) {
        return "¥ ${(num / 1000000).toStringAsFixed(1)} M";
      } else if (num > 999999999) {
        return "¥ ${(num / 1000000000).toStringAsFixed(1)} B";
      } else {
        return "¥ " + num.toString();
      }
    } else if (countryCode == "eu") {
      if (num > 999 && num < 99999) {
        return "€ ${(num / 1000).toStringAsFixed(1)} K";
      } else if (num > 99999 && num < 999999) {
        return "€ ${(num / 1000).toStringAsFixed(0)} K";
      } else if (num > 999999 && num < 999999999) {
        return "€ ${(num / 1000000).toStringAsFixed(1)} M";
      } else if (num > 999999999) {
        return "€ ${(num / 1000000000).toStringAsFixed(1)} B";
      } else {
        return "€ " + num.toString();
      }
    } else if (countryCode == "uk") {
      if (num > 999 && num < 99999) {
        return "£ ${(num / 1000).toStringAsFixed(1)} K";
      } else if (num > 99999 && num < 999999) {
        return "£ ${(num / 1000).toStringAsFixed(0)} K";
      } else if (num > 999999 && num < 999999999) {
        return "£ ${(num / 1000000).toStringAsFixed(1)} M";
      } else if (num > 999999999) {
        return "£ ${(num / 1000000000).toStringAsFixed(1)} B";
      } else {
        return "£ " + num.toString();
      }
    } else {
      if (num > 999 && num < 99999) {
        return "${(num / 1000).toStringAsFixed(1)}";
      } else if (num > 99999 && num < 999999) {
        return "${(num / 1000).toStringAsFixed(0)}";
      } else if (num > 999999 && num < 999999999) {
        return "${(num / 1000000).toStringAsFixed(1)}";
      } else if (num > 999999999) {
        return "${(num / 1000000000).toStringAsFixed(1)}";
      } else {
        return num.toString();
      }
    }
  }
}
